import brian_no_units
from brian import *
set_global_preferences(useweave=True)
import cuba_runopts
cuba_runopts.cuba_opts['connections'] = False
import cuba
cuba.do_runs('data/brian_cuba_nospiking_results_compile_nounits.pkl')



'''
Realtime Brian

Pauses are inserted so that Brian runs always between real time and
about 50 ms ahead of it (probably a bit more).
Principle: every 50 ms, we check whether Brian is ahead by more than
100 ms, and if so we insert a sleep() to synchronize.

NB: of course Brian needs to be faster than real time!
(so use a large timestep)

Other features of this real time stuff:
* set the offset of the real time clock (reference time)
'''
from brian import *
from time import time,sleep

defaultclock.dt=0.5*ms

G=NeuronGroup(1000,model='dv/dt=-v/(10*ms):1')

first_time=True
@network_operation(EventClock(dt=50*ms))
def catch_up(cl):
    global start_time,first_time
    # First time: synchronize Brian and real time
    if first_time:
        start_time=time()
        first_time=False
    real_time=time()-start_time
    print cl.t,real_time
    if cl._t>real_time:
        sleep(cl._t-real_time)
    
run(10*second)



import cloud
from keys import *
from numpy import array, cumsum, nonzero


naccounts = len(api_keys)
max_parallel = sum(array(parallelism[:naccounts]))
print "Current parallelism level : %d" % max_parallel

__all__ = ['multimap', 'retrieve', 'status', 'naccounts', 'max_parallel']

def multimap(fun, args, naccounts=None):
    if naccounts is None:
        naccounts = len(api_keys)

    max_parallel = sum(array(parallelism[:naccounts]))
    if len(args) <= max_parallel:
        naccounts = nonzero(cumsum(parallelism) - len(args) >= 0)[0][0] + 1
        size = parallelism[:naccounts]
    else:
        size = [len(args) / naccounts for _ in xrange(naccounts)]
    if naccounts > 1:
        size[-1] = len(args) - sum(array(size[:-1]))
    else:
        size[0] = len(args)

    # jids[i] contains the job indices for account i
    jids = [None for _ in xrange(naccounts)]

    # Launches the jobs
    k = 0
    for i in xrange(naccounts):
        api_key = api_keys[i]
        api_secretkey = api_secretkeys[i]
        n = size[i]

        args_tmp = args[k:k + n]
        if len(args_tmp) > 0:
            print "Launching %d jobs with account %d..." % (len(args_tmp), i + 1)
            cloud.setkey(api_key=api_key, api_secretkey=api_secretkey)
            jids[i] = cloud.map(fun, args_tmp, _high_cpu=True)
            print "    Jobs:", jids[i]
            k += n
    return jids

def retrieve(jids):
    naccounts = len(jids)
    results = []
    # Retrieves the results
    for i in xrange(naccounts):
        api_key = api_keys[i]
        api_secretkey = api_secretkeys[i]
        cloud.setkey(api_key=api_key, api_secretkey=api_secretkey)
        print "Retrieving results for account %d..." % (i + 1)
        results.extend(cloud.result(jids[i]))

    return results

def status(jids):
    naccounts = len(jids)
    status = []
    # Retrieves the results
    for i in xrange(naccounts):
        api_key = api_keys[i]
        api_secretkey = api_secretkeys[i]
        cloud.setkey(api_key=api_key, api_secretkey=api_secretkey)
        print "Retrieving status for account %d..." % (i + 1)
        status.extend(cloud.status(jids[i]))

    return status



from scipy import *
from time import time, clock

__all__ = ['particle_swarm']

def particle_swarm(X0, fun, iterations, pso_params, min_values=None, max_values=None, group_size=None, verbose=True, return_matrix=None):
    """
    Computes the argument of fun which maximizes it using the Particle Swarm Optimization algorithm.
    
    INPUTS
    X0 is a N*M matrix
    N is the space dimension
    M is the number of particles
    If min_values is set, it is a N-long array such that X >= min_values.
    If max_values is set, it is a N-long array such that X <= max_values.
    fun(x0) is a 1*M row vector
    If group_size is set, it means that particles are grouped within groups of size group_size
    The objective function is assumed to make a different computation for each group.
    The PSO algorithm then maximizes f in each group independently. It returns a matrix
    of size N*M/group_size : each column is the result of the optimization for the corresponding group.
    
    OUTPUTS
    pso(x0, fun) returns argmax fun(x)
    """

    (N, M) = X0.shape
    if group_size is None:
        group_size = M
    group_number = M / group_size

    if (min_values is None):
        min_values = -inf * ones(N)

    if (max_values is None):
        max_values = inf * ones(N)

    min_values = tile(min_values.reshape((-1, 1)), (1, M))
    max_values = tile(max_values.reshape((-1, 1)), (1, M))

    fitness_X = -inf * ones((1, M))
    fitness_lbest = fitness_X
    fitness_gbest = -inf * ones(group_number)

    omega = pso_params[0]
    c1 = pso_params[1]
    c2 = pso_params[2]

    X0 = maximum(X0, min_values)
    X0 = minimum(X0, max_values)
    X = X0
    V = zeros((N, M))

    X_lbest = X
    X_gbest = X[:, arange(0, M, group_size)] # N*group_number matrix

    if return_matrix:
        fitness_X_matrix = zeros((M, iterations))

    time0 = clock()
    for k in range(iterations):
        if verbose:
            print 'Step %d/%d' % (k + 1 , iterations)
        R1 = rand(N, M)
        R2 = rand(N, M)
        X_gbest2 = zeros((N, M))
        for j in range(group_number):
            X_gbest2[:, j * group_size:(j + 1) * group_size] = tile(X_gbest[:, j].reshape(N, 1), (1, group_size))
        V = omega * V + c1 * R1 * (X_lbest - X) + c2 * R2 * (X_gbest2 - X)
        X = X + V

        X = maximum(X, min_values)
        X = minimum(X, max_values)

        time1 = clock()
        fitness_X = fun(X)
        time2 = clock()
        if return_matrix:
            fitness_X_matrix[:, k] = fitness_X

        # Local update
        indices_lbest = nonzero(fitness_X > fitness_lbest)[1]
        if (len(indices_lbest) > 0):
            X_lbest[:, indices_lbest] = X[:, indices_lbest]
            fitness_lbest[:, indices_lbest] = fitness_X[:, indices_lbest]

        # Global update
        max_fitness_X = array([max(fitness_X[j * group_size:(j + 1) * group_size]) for j in range(group_number)])
        for j in nonzero(max_fitness_X > fitness_gbest)[0]: # groups for which a global best has been reached at this iteration
            sub_fitness_X = fitness_X[j * group_size:(j + 1) * group_size]
            index_gbest = nonzero(sub_fitness_X == max_fitness_X[j])[0]
            if not(isscalar(index_gbest)):
                index_gbest = index_gbest[0]
            X_gbest[:, j] = X[:, j * group_size + index_gbest]
            fitness_gbest[j] = max_fitness_X[j]

        if verbose:
            print 'Evaluation time :', '%.2f s' % (time2 - time1)
            print 'Total elapsed time :', '%.2f s' % (clock() - time0)
            print 'Fitness: mean = %.3f, max = %.3f, std = %.3f' % (mean(fitness_X), max(fitness_X), std(fitness_X))
#            print '  min  :', '%.5f' % fitness_gbest.min()
#            print '  mean :', '%.5f' % fitness_gbest.mean()
#            print '  max  :', '%.5f' % fitness_gbest.max()
#            print '  std  :', '%.5f' % fitness_gbest.std()
#        for j in range(group_number):
#            print '  %d :' % (j+1), '%.5f' % fitness_gbest[j]
        print

    if return_matrix:
        return(X_gbest, fitness_gbest, clock() - time0, fitness_X_matrix)
    else:
        return (X_gbest, fitness_gbest, clock() - time0)



if __name__ == "__main__":
    group_number = 5
    group_size = 10

    def fun(x):
        y = exp(-.7 * sum(x ** 2, axis=0))
        return y
    result = optimize(X0=3 * rand(5, group_size * group_number), fun=fun, iterations=500, pso_params=[.8, 1.8, 1.8], group_size=group_size)

#    print '-------------------------------'
#    print '-------------------------------'
#    print 'Final value : %.10f' % result



from brian import *

N = 10

burst_clock = EventClock(dt=100 * ms)

def burst_generator():
    start = burst_clock.t
    times = [(i, start + rand()*10 * ms) for i in range(N)]
    times.sort(key=lambda (i, t):t)
    return times

@network_operation(clock=burst_clock)
def next_burst():
    G.reinit()

G = SpikeGeneratorGroup(N, burst_generator)

M = SpikeMonitor(G)
run(500 * ms)
raster_plot(M)
show()



'''
Spike statistics
----------------
In all functions below, spikes is a sorted list of spike times
'''
from numpy import *
from brian.units import check_units, second
from brian.stdunits import ms, Hz
from operator import itemgetter

__all__ = ['firing_rate', 'CV', 'correlogram', 'autocorrelogram', 'CCF', 'ACF', 'CCVF', 'ACVF', 'group_correlations', 'sort_spikes',
         'total_correlation', 'vector_strength', 'gamma_factor', 'get_gamma_factor_matrix', 'get_gamma_factor','spike_triggered_average']

# First-order statistics
def firing_rate(spikes):
    '''
    Rate of the spike train.
    '''
    if spikes==[]:
        return NaN
    return (len(spikes) - 1) / (spikes[-1] - spikes[0])

def CV(spikes):
    '''
    Coefficient of variation.
    '''
    if spikes==[]:
        return NaN
    ISI = diff(spikes) # interspike intervals
    return std(ISI) / mean(ISI)

# Second-order statistics
def correlogram(T1, T2, width=20 * ms, bin=1 * ms, T=None):
    '''
    Returns a cross-correlogram with lag in [-width,width] and given bin size.
    T is the total duration (optional) and should be greater than the duration of T1 and T2.
    The result is in Hz (rate of coincidences in each bin).

    N.B.: units are discarded.
    TODO: optimise?
    '''
    if (T1==[]) or (T2==[]): # empty spike train
        return NaN
    # Remove units
    width = float(width)
    T1 = array(T1)
    T2 = array(T2)
    i = 0
    j = 0
    n = int(ceil(width / bin)) # Histogram length
    l = []
    for t in T1:
        while i < len(T2) and T2[i] < t - width: # other possibility use searchsorted
            i += 1
        while j < len(T2) and T2[j] < t + width:
            j += 1
        l.extend(T2[i:j] - t)
    H, _ = histogram(l, bins=arange(2 * n + 1) * bin - n * bin) #, new = True)

    # Divide by time to get rate
    if T is None:
        T = max(T1[-1], T2[-1]) - min(T1[0], T2[0])
    # Windowing function (triangle)
    W = zeros(2 * n)
    W[:n] = T - bin * arange(n - 1, -1, -1)
    W[n:] = T - bin * arange(n)

    return H / W

def autocorrelogram(T0, width=20 * ms, bin=1 * ms, T=None):
    '''
    Returns an autocorrelogram with lag in [-width,width] and given bin size.
    T is the total duration (optional) and should be greater than the duration of T1 and T2.
    The result is in Hz (rate of coincidences in each bin).

    N.B.: units are discarded.
    '''
    return correlogram(T0, T0, width, bin, T)

def CCF(T1, T2, width=20 * ms, bin=1 * ms, T=None):
    '''
    Returns the cross-correlation function with lag in [-width,width] and given bin size.
    T is the total duration (optional).
    The result is in Hz**2:
    CCF(T1,T2)=<T1(t)T2(t+s)>

    N.B.: units are discarded.
    '''
    return correlogram(T1, T2, width, bin, T) / bin

def ACF(T0, width=20 * ms, bin=1 * ms, T=None):
    '''
    Returns the autocorrelation function with lag in [-width,width] and given bin size.
    T is the total duration (optional).
    The result is in Hz**2:
    ACF(T0)=<T0(t)T0(t+s)>

    N.B.: units are discarded.
    '''
    return CCF(T0, T0, width, bin, T)

def CCVF(T1, T2, width=20 * ms, bin=1 * ms, T=None):
    '''
    Returns the cross-covariance function with lag in [-width,width] and given bin size.
    T is the total duration (optional).
    The result is in Hz**2:
    CCVF(T1,T2)=<T1(t)T2(t+s)>-<T1><T2>

    N.B.: units are discarded.
    '''
    return CCF(T1, T2, width, bin, T) - firing_rate(T1) * firing_rate(T2)

def ACVF(T0, width=20 * ms, bin=1 * ms, T=None):
    '''
    Returns the autocovariance function with lag in [-width,width] and given bin size.
    T is the total duration (optional).
    The result is in Hz**2:
    ACVF(T0)=<T0(t)T0(t+s)>-<T0>**2

    N.B.: units are discarded.
    '''
    return CCVF(T0, T0, width, bin, T)


def spike_triggered_average(spikes,stimulus,max_interval,dt,onset=None,display=False):
    '''
    Spike triggered average reverse correlation. 
    spikes is an array containing spike times
    stimulus is an array containing the stimulus
    max_interval (second) is the duration of the averaging window
    dt (second) is the sampling period
    onset (second) before which the spikes are discarded. Note: it will be at least as long as max_interval
    display (default=False) display the number of spikes processed out of the total number
    output the spike triggered average and the corresponding time axis
    '''
    stimulus = stimulus.flatten()
    if onset < max_interval or onset == None:
        onset = max_interval
    nspikes = len(spikes)
    sta_length = int(max_interval/dt)
    spike_triggered_ensemble=zeros((nspikes,sta_length))
    time_axis = linspace(0*ms,max_interval,sta_length)
    onset = float(onset)
    for ispike,spike in enumerate(spikes):
        if display==True:
            print 'sta: spike #',ispike,' out of :',nspikes
        if spike>onset:
            spike = int(spike/dt) 
            print stimulus[spike-sta_length:spike].shape
            spike_triggered_ensemble[ispike,:] = stimulus[spike-sta_length:spike]
            ispike +=1
            
    return sum(spike_triggered_ensemble,axis=0)[::-1]/(nspikes-1),time_axis



def total_correlation(T1, T2, width=20 * ms, T=None):
    '''
    Returns the total correlation coefficient with lag in [-width,width].
    T is the total duration (optional).
    The result is a real (typically in [0,1]):
    total_correlation(T1,T2)=int(CCVF(T1,T2))/rate(T1)
    '''
    if (T1==[]) or (T2==[]): # empty spike train
        return NaN
    # Remove units
    width = float(width)
    T1 = array(T1)
    T2 = array(T2)
    # Divide by time to get rate
    if T is None:
        T = max(T1[-1], T2[-1]) - min(T1[0], T2[0])
    i = 0
    j = 0
    x = 0
    for t in T1:
        while i < len(T2) and T2[i] < t - width: # other possibility use searchsorted
            i += 1
        while j < len(T2) and T2[j] < t + width:
            j += 1
        x += sum(1. / (T - abs(T2[i:j] - t))) # counts coincidences with windowing (probabilities)
    return float(x / firing_rate(T1)) - float(firing_rate(T2) * 2 * width)

def sort_spikes(spikes):
    """
    Sorts spikes stored in a (i,t) list by time.
    """
    spikes = sorted(spikes, key=itemgetter(1))
    return spikes

def group_correlations(spikes, delta=None):
    """
    Computes the pairwise correlation strength and timescale of the given pool of spike trains.
    spikes is a (i,t) list and must be sorted.
    delta is the length of the time window, 10*ms by default.
    """
    aspikes = array(spikes)
    N = aspikes[:, 0].max() + 1 # neuron count
    T = aspikes[:, 1].max() # total duration
    spikecount = zeros(N)
    tauc = zeros((N, N))
    S = zeros((N, N))
    if delta is None:
        delta = 10 * ms # size of the window
    windows = -2 * delta * ones(N) # windows[i] is the end of the window for neuron i = lastspike[i}+delta
    for i, t in spikes:
        sources = (t <= windows) # neurons such that (i,t) is a target spike for them
        if sum(sources) > 0:
            indices = nonzero(sources)[0]
            S[indices, i] += 1
            delays = t - windows[indices] + delta
#            print i, t, indices, delays
            tauc[indices, i] += delays
        spikecount[i] += 1
        windows[i] = t + delta # window update

    tauc /= S

    S = S / tile(spikecount.reshape((-1, 1)), (1, N)) # normalize S
    rates = spikecount / T
    S = S - tile(rates.reshape((1, -1)), (N, 1)) * delta

    S[isnan(S)] = 0.0
    tauc[isnan(tauc)] = 0.0

    return S, tauc

# Phase-locking properties
def vector_strength(spikes, period):
    '''
    Returns the vector strength of the given train
    '''
    return abs(mean(exp(array(spikes) * 1j * 2 * pi / period)))

# Normalize the coincidence count of two spike trains (return the gamma factor)
def get_gamma_factor(coincidence_count, model_length, target_length, target_rates, delta):
    NCoincAvg = 2 * delta * target_length * target_rates
    norm = .5 * (1 - 2 * delta * target_rates)
    gamma = (coincidence_count - NCoincAvg) / (norm * (target_length + model_length))
    return gamma

# Normalize the coincidence matrix between a set of  trains (return the gamma factor matrix)
def get_gamma_factor_matrix(coincidence_matrix, model_length, target_length, target_rates, delta):

    target_lengthMAT =tile(target_length,(len(model_length),1))
    target_rateMAT =tile(target_rates,(len(model_length),1))
    model_lengthMAT  =tile(model_length.reshape(-1,1),(1,len(target_length)))
    NCoincAvg  =2 * delta * target_lengthMAT * target_rateMAT 
    norm =.5 * (1 - 2 * delta * target_rateMAT)

   # print  target_rateMAT 
    print coincidence_matrix 
    #print NCoincAvg
    #print (norm * (target_lengthMAT + model_lengthMAT))
    gamma = (coincidence_matrix - NCoincAvg) / (norm * (target_lengthMAT + model_lengthMAT))
    gamma=triu(gamma,0)+triu(gamma,1).T
    return gamma


# Gamma factor
@check_units(delta=second)
def gamma_factor(source, target, delta, normalize=True, dt=None):
    '''
    Returns the gamma precision factor between source and target trains,
    with precision delta.
    source and target are lists of spike times.
    If normalize is True, the function returns the normalized gamma factor 
    (less than 1.0), otherwise it returns the number of coincidences.
    dt is the precision of the trains, by default it is defaultclock.dt
    
    Reference:
    R. Jolivet et al., 'A benchmark test for a quantitative assessment of simple neuron models',
        Journal of Neuroscience Methods 169, no. 2 (2008): 417-424.
    '''

    source = array(source)
    target = array(target)
    target_rate = firing_rate(target) * Hz

    if dt is None:
        delta_diff = delta
    else:
        source = array(rint(source / dt), dtype=int)
        target = array(rint(target / dt), dtype=int)
        delta_diff = int(rint(delta / dt))

    source_length = len(source)
    target_length = len(target)

    if (target_length == 0 or source_length == 0):
        return 0

    if (source_length > 1):
        bins = .5 * (source[1:] + source[:-1])
        indices = digitize(target, bins)
        diff = abs(target - source[indices])
        matched_spikes = (diff <= delta_diff)
        coincidences = sum(matched_spikes)
    else:
        indices = [amin(abs(source - target[i])) <= delta_diff for i in xrange(target_length)]
        coincidences = sum(indices)

    # Normalization of the coincidences count
#    NCoincAvg = 2 * delta * target_length * target_rate
#    norm = .5*(1 - 2 * target_rate * delta)
#    gamma = (coincidences - NCoincAvg)/(norm*(source_length + target_length))

    # TODO: test this
    gamma = get_gamma_factor(coincidences, source_length, target_length, target_rate, delta)

    if normalize:
        return gamma
    else:
        return coincidences

if __name__ == '__main__':

    from brian import *

    print vector_strength([1.1 * ms, 1 * ms, .9 * ms], 2 * ms)

    N = 100000
    T1 = cumsum(rand(N) * 10 * ms)
    T2 = cumsum(rand(N) * 10 * ms)
    duration = T1[N / 2] # Cut so that both spike trains have the same duration
    T1 = T1[T1 < duration]
    T2 = T2[T2 < duration]
    print firing_rate(T1)
    C = CCVF(T1, T2, bin=1 * ms)
    print total_correlation(T1, T2)
    plot(C)
    show()



from brian import *
set_global_preferences(usenewbrianhears=True)
from brian.hears import *

hrtfdb = IRCAM_LISTEN(r'F:\HRTF\IRCAM', compensated=True)
hrtfset = hrtfdb.load_subject(1002)

print amin(hrtfset.coordinates['elev'])

print len(hrtfset)

hrtfset = hrtfset.subset(lambda elev:abs(elev)<=15)

print len(hrtfset)

hrtf = hrtfset(azim=30, elev=15)

plot(hrtf.left)
plot(hrtf.right)
show()



from brian import *
from filterbank import Filterbank,FunctionFilterbank,ControlFilterbank, CombinedFilterbank
from filterbanklibrary import *
from linearfilterbank import *
import warnings
from scipy.io import loadmat,savemat
from brian.hears import *

__all__=['TanCarney']

try:
    from scikits.samplerate import resample
    have_scikits_samplerate = True
except (ImportError, ValueError):
    have_scikits_samplerate = False
#print have_scikits_samplerate

def set_parameters(cf,param):
    
    parameters=dict()
    parameters['fc_LP_control']=800 #Hz
    parameters['fc_LP_fb']=500 #Hz
    parameters['fp1']=1.0854*cf-106.0034
    parameters['ta']=10**(log10(cf)*1.0230 + 0.1607)
    parameters['tb']=10**(log10(cf)*1.4292 - 1.1550) - 1000
    parameters['gain80']=10**(log10(cf)*0.5732 + 1.5220)
    parameters['rgain']=10**( log10(cf)*0.4 + 1.9)
    parameters['average_control']=0.3357
    parameters['zero_r']= array(-10**( log10(cf)*1.5-0.9 ))   
        
    if param: 
        if not isinstance(param, dict): 
            raise Error('given parameters must be a dict')
        for key in param.keys():
            if not parameters.has_key(key):
                raise Exception(key + ' is invalid key entry for given parameters')
            parameters[key] = param[key]
    parameters['nlgain']= (parameters['gain80'] - parameters['rgain'])/parameters['average_control']
    return parameters


class Control_Coefficients:
    
    def __init__(self,cf,samplerate):
        self.cf = cf
        self.PI2 = 2*3.14159265358979
        self.nch=len(cf)
        self.fs_bilinear = 2.0*samplerate#*ones(self.nch)
#        self.fs_bilinear =tile(self.fs_bilinear.reshape(self.nch,-1),3)
        self.x_cf=11.9*log10(0.8+cf/456);
        self.f_shift=(pow(10,((self.x_cf+1.2)/11.9))-0.8)*456-cf
        self.wbw=cf/4.0
        self.filt_a = zeros((len(cf),3,4), order='F')
#        self.filt_a[:,0,:] = 1
        self.filt_b = zeros((len(cf),3,4), order='F')
        self.control_signal = 0
        self.preal = zeros((self.nch,6))
        self.pimg = zeros((self.nch,6))
        self.preal,self.pimg = self.analog_poles()
        
    def return_coefficients(self,control_signal):

        self.wbw=-(self.preal[:,0] - control_signal)/self.PI2
        self.gain_norm_bp=((self.PI2**2*sqrt(self.wbw**2 + self.f_shift**2)*sqrt( (2*self.cf+self.f_shift)**2 + self.wbw**2 ))**3)/sqrt(self.PI2**2*self.cf**2)
        self.gain_norm_bp =tile(self.gain_norm_bp.reshape(self.nch,-1),3)

        iord = [1,3,5]
        temp=(self.fs_bilinear-self.preal[:,iord])**2 + self.pimg[:,iord]**2
        self.filt_a[:,0,0:3] = 1.
        self.filt_a[:,1,0:3]= -2*(self.fs_bilinear**2-self.preal[:,iord]**2-self.pimg[:,iord]**2)/temp            
        self.filt_a[:,2,0:3] = ((self.fs_bilinear+self.preal[:,iord])**2+self.pimg[:,iord]**2)/temp
        self.filt_b[:,0,0:3] = 1./temp
        self.filt_b[:,1,0:3] = 2./temp  
        self.filt_b[:,2,0:3] = 1./temp
        
        self.filt_a[:,0,3] = 1.
        self.filt_a[:,1,3]= 1.      
        self.filt_a[:,2,3] = 0.
        self.filt_b[:,0,3] = self.fs_bilinear
        self.filt_b[:,1,3] = -self.fs_bilinear
        self.filt_b[:,2,3] = 0
        
        self.filt_b[:,:,3] = self.gain_norm_bp*self.filt_b[:,:,3]  
        return self.filt_b,self.filt_a

    def analog_poles(self):
        self.preal[:,0] = self.PI2*self.wbw
        self.preal[:,1] = -self.PI2*self.wbw
        self.preal[:,2] = self.preal[:,0]
        self.preal[:,3] = self.preal[:,1]
        self.preal[:,4] = self.preal[:,0]
        self.preal[:,5] = self.preal[:,1]

        self.pimg[:,0] = self.PI2*(self.cf+self.f_shift)
        self.pimg[:,1] = -self.PI2*(self.cf+self.f_shift)
        self.pimg[:,2] = self.pimg[:,0]
        self.pimg[:,3] = self.pimg[:,1]
        self.pimg[:,4] = self.pimg[:,0]
        self.pimg[:,5] = self.pimg[:,1]
        
        return self.preal,self.pimg
    
    
class Signal_Coefficients:
    
    def __init__(self,cf,samplerate,parameters):
        self.cf = cf
        self.PI2 = 2*3.14159265358979
        self.nch=len(cf)
        self.fs_bilinear = 2.0*samplerate#*ones(self.nch)
        
        self.order_of_pole = 20             
        self.half_order_pole = self.order_of_pole/2
        self.order_of_zero  = self.half_order_pole

        self.filt_a = zeros((len(cf),3,10), order='F')
        self.filt_b = zeros((len(cf),3,10), order='F')
        
        self.preal = zeros((self.nch,20))
        self.pimg = zeros((self.nch,20))
        
        self.control_signal = 0
        
        self.rgain =parameters['rgain']# 10**(log10(cf)*0.4 + 1.9)
        self.fp1=parameters['fp1']#1.0854*cf-106.0034
        self.ta= parameters['ta']#    10**(log10(cf)*1.0230 + 0.1607)
        self.tb= parameters['tb']#    10**(log10(cf)*1.4292 - 1.1550) - 1000
        self.zeroa = parameters['zero_r']# -10**(log10(cf)*1.5-0.9 )  
        
        self.zeroamat = tile(self.zeroa.reshape(self.nch,-1),10)
        self.preal,self.pimg = self.analog_poles(0)

        
        self.cfmat = tile(self.cf.reshape(self.nch,-1),20)  
        self.gain_norm = sqrt(prod((2*pi*self.cfmat-self.pimg[:,0:20])**2+self.preal[:,0:20]**2,axis=1))

        self.gain_norm = self.gain_norm /(sqrt((2*pi*self.cf)**2+self.zeroa**2))**self.order_of_zero
        self.gain_norm  = tile(self.gain_norm.reshape(self.nch,-1),3)

        
    def return_coefficients(self,control_signal):
        self.preal,self.pimg = self.analog_poles(control_signal)
        
        iord = arange(2,22,2)-1
        temp=(self.fs_bilinear-self.preal[:,iord])**2 + self.pimg[:,iord]**2
    
        self.filt_a[:,0,:] = 1
        self.filt_a[:,1,:] = -2*(self.fs_bilinear**2-self.preal[:,iord]**2-self.pimg[:,iord]**2)/temp            
        self.filt_a[:,2,:] = ((self.fs_bilinear+self.preal[:,iord])**2+self.pimg[:,iord]**2)/temp
        
        self.filt_b[:,0,:] = (-self.zeroamat+self.fs_bilinear)/temp
        self.filt_b[:,1,:] = (-2*self.zeroamat)/temp
        self.filt_b[:,2,:] = (-self.zeroamat-self.fs_bilinear)/temp
        self.filt_b[:,:,9] = self.gain_norm/3.*self.filt_b[:,:,9] 

        return self.filt_b,self.filt_a

    def analog_poles(self,control_signal):
        self.preal[:,0] = -self.rgain-control_signal
        self.preal[:,4] = self.preal[:,0]-self.ta
        self.preal[:,2] = (self.preal[:,0]+self.preal[:,4])*0.5
        self.preal[:,1] = self.preal[:,0]
        self.preal[:,3] = self.preal[:,2]
        self.preal[:,5] = self.preal[:,4]
        
        self.preal[:,6] = self.preal[:,0]
        self.preal[:,7] = self.preal[:,1]
        self.preal[:,8] = self.preal[:,4]
        self.preal[:,9] = self.preal[:,5]
        self.preal[:,10:] = self.preal[:,:10]
        
        self.pimg[:,0] = self.PI2*self.fp1
        self.pimg[:,4] = self.pimg[:,0]-self.tb
        self.pimg[:,2] = (self.pimg[:,0]+self.pimg[:,4])*0.5
        self.pimg[:,1] = -self.pimg[:,0]
        self.pimg[:,3] = -self.pimg[:,2]
        self.pimg[:,5] = -self.pimg[:,4]
        self.pimg[:,6] = self.pimg[:,0]
        self.pimg[:,7] = self.pimg[:,1]
        self.pimg[:,8] = self.pimg[:,4]
        self.pimg[:,9] = self.pimg[:,5]
        self.pimg[:,10:] = self.pimg[:,:10]

        return self.preal,self.pimg
    
class Filter_Update: 
    def __init__(self, target,coef):
        self.coef = coef
        self.target = target
        self.param = []
    def __call__(self,input):  
        self.target.filt_b,self.target.filt_a = self.coef.return_coefficients(input[-1,:]) 
        self.param.append(self.coef.control_signal)


class LowPass_IHC(LinearFilterbank):
    def __init__(self,source,cf,fc,gain,order): 
        nch = len(cf)
        TWOPI = 2*pi
        self.samplerate =  source.samplerate
        c = 2.0 * self.samplerate
        c1LP = ( c/Hz - TWOPI*fc ) / ( c/Hz + TWOPI*fc )
        c2LP = TWOPI*fc/Hz / (TWOPI*fc + c/Hz)
        
        b_temp = array([c2LP,c2LP])
        a_temp = array([1,-c1LP])
        
        filt_b = tile(b_temp.reshape([2,1]),[nch,1,order])               
        filt_a = tile(a_temp.reshape([2,1]),[nch,1,order]) 
        filt_b[:,:,0] = filt_b[:,:,0]*gain

        LinearFilterbank.__init__(self, source, filt_b, filt_a)

class LowPass_filter(LinearFilterbank):
    def __init__(self,source,cf,fc,gain,order):
        nch = len(cf)
        TWOPI = 2*pi
        self.samplerate =  source.samplerate
        c = 2.0 * self.samplerate
        c1LP = ( c/Hz - TWOPI*fc ) 
        
        b_temp = array([1,1])/ ( c/Hz + TWOPI*fc )
        a_temp = array([1,-c1LP/ ( c/Hz + TWOPI*fc )])
        
        filt_b = tile(b_temp.reshape([2,1]),[nch,1,order])               
        filt_a = tile(a_temp.reshape([2,1]),[nch,1,order]) 
        filt_b[:,:,order-1] = filt_b[:,:,order-1]*gain

        LinearFilterbank.__init__(self, source, filt_b, filt_a)

def saturation_fc(x,A0=1,B=1,C=1,D=1):
    ind = x>=0
    x[ind]=A0*log(x[ind]*B+1.0)
    ind = x<0
    dtemp = (-x[ind])**C
    tempA = -A0*(dtemp+D)/(3*dtemp+D);
    x[ind]=tempA*log(abs(x[ind])*B+1.0)

    return x
    

class TanCarney(CombinedFilterbank):
    '''
    Class implementing the nonlinear auditory filterbank model as described in
    Tan, G. and Carney, L., 
    "A phenomenological model for the responses of auditory-nerve
    fibers. II. Nonlinear tuning with a frequency glide", JASA 2003.
    
    The model consists of a control path and a signal path. The control path
    controls both its own bandwidth via a feedback
    loop and also the bandwidth of the signal path. 
    
    Initialised with arguments:
    
    ``source``
        Source of the cochlear model.
        
    ``cf``
        List or array of center frequencies.
        
    ``update_interval``
        Interval in samples controlling how often the band pass filter of the
        signal pathway is updated. Smaller values are more accurate but
        increase the computation time.
        
    ``param``
        Dictionary used to overwrite the default parameters given in the
        original paper. 
    '''
    
    def __init__(self, source,cf,update_interval,param={}):

        CombinedFilterbank.__init__(self, source)
        source = self.get_modified_source()       
        cf = atleast_1d(cf)
        nbr_cf=len(cf)
        samplerate=source.samplerate

        parameters=set_parameters(cf,param)
        if int(source.samplerate)!=50000:
            warnings.warn('To use the TanCarney cochlear model the sample rate should be 50kHz')
#            if not have_scikits_samplerate:
#                raise ImportError('To use the PMFR cochlear model the sample rate should be 50kHz and scikits.samplerate package is needed for resampling')               
#            #source=source.resample(50*kHz)
#            warnings.warn('The input to the PMFR cochlear model has been resampled to 50kHz'
          
#        ##### Control Path ####
        # band pass filter
        control_coef = Control_Coefficients(cf, samplerate)
        [filt_b,filt_a] = control_coef.return_coefficients(0)
        BP_control = LinearFilterbank(source,filt_b,filt_a)        
        # first non linearity of control path
        Acp,Bcp,Ccp=100.,2.5,0.60 
        func_NL1_control=lambda x:sign(x)*Bcp*log(1.+Acp*abs(x)**Ccp)
        NL1_control=FunctionFilterbank(BP_control,func_NL1_control)        
        # second non linearity of control path
        asym,s0,x1,s1=7.,8.,5.,3. 
        shift = 1./(1.+asym)
        x0 = s0*log((1.0/shift-1)/(1+exp(x1/s1)))
        func_NL2_control=lambda x:(1.0/(1.0+exp(-(x-x0)/s0)*(1.0+exp(-(x-x1)/s1)))-shift)*parameters['nlgain']
        NL2_control=FunctionFilterbank(NL1_control,func_NL2_control)       
        #control low pass filter (its output will be used to control the signal path)
        gain_lp_con = (2*pi*parameters['fc_LP_control'])**3*1.5
        LP_control = LowPass_filter(NL2_control,cf,parameters['fc_LP_control'],gain_lp_con,3)   
        #low pass filter for feedback to control band pass (its output will be used to control the control path)
        gain_lp_fb = parameters['fc_LP_fb']*2*pi *10
        LP_feed_back = LowPass_filter(LP_control,cf,parameters['fc_LP_fb'],gain_lp_fb,1) 
         
         
        #### signal path  ####
        # band pass filter
        signal_coef = Signal_Coefficients(cf, samplerate,parameters)
        [filt_b,filt_a] = signal_coef.return_coefficients(0)
        BP_signal = LinearFilterbank(source,filt_b,filt_a)
        ## Saturation
        saturation = FunctionFilterbank(BP_signal,saturation_fc,A0=0.1,B=2000,C=1.74,D=6.87e-9)   
        ## low pass IHC     
        ihc = LowPass_IHC(saturation,cf,3800,1,7)
        
        ### controlers ###
        updater1=Filter_Update(BP_control,control_coef) #instantiation of the updater for the control path
        output1 = ControlFilterbank(ihc, LP_feed_back, BP_control,updater1, update_interval)  #controler for the band pass filter of the control path
        
        updater2=Filter_Update(BP_signal,signal_coef) #instantiation of the updater for the control path
        output2 = ControlFilterbank(output1, LP_control, BP_signal,updater2, update_interval)  #controler for the band pass filter of the control path
        
#
#        self.control_cont = updater1.param
#        self.signal_cont = updater2.param
#        self.set_output(BP_control)
        self.set_output(output2)

    


'''
Spike-timing dependent plasticity
Adapted from Song, Miller and Abbott (2000) and Song and Abbott (2001)

This simulation takes a long time!
'''
from brian import *
from time import time

N = 1000
taum = 10 * ms
tau_pre = 20 * ms
tau_post = tau_pre
Ee = 0 * mV
vt = -54 * mV
vr = -60 * mV
El = -74 * mV
taue = 5 * ms
F = 15 * Hz
gmax = .01
dA_pre = .01
dA_post = -dA_pre * tau_pre / tau_post * 1.05

eqs_neurons = '''
dv/dt=(ge*(Ee-vr)+El-v)/taum : volt   # the synaptic current is linearized
dge/dt=-ge/taue : 1
'''

input = PoissonGroup(N, rates=F)
neurons = NeuronGroup(1, model=eqs_neurons, threshold=vt, reset=vr)
synapses = Connection(input, neurons, 'ge', weight=rand(len(input), len(neurons)) * gmax)
neurons.v = vr

#stdp=ExponentialSTDP(synapses,tau_pre,tau_post,dA_pre,dA_post,wmax=gmax)
## Explicit STDP rule
eqs_stdp = '''
dA_pre/dt=-A_pre/tau_pre : 1
dA_post/dt=-A_post/tau_post : 1
'''
dA_post *= gmax
dA_pre *= gmax
stdp = STDP(synapses, eqs=eqs_stdp, pre='A_pre+=dA_pre;w+=A_post',
          post='A_post+=dA_post;w+=A_pre', wmax=gmax)

rate = PopulationRateMonitor(neurons)

start_time = time()
run(100 * second, report='text')
print "Simulation time:", time() - start_time

subplot(311)
plot(rate.times / second, rate.smooth_rate(100 * ms))
subplot(312)
plot(synapses.W.todense() / gmax, '.')
subplot(313)
hist(synapses.W.todense() / gmax, 20)
show()



'''
This demonstrates the 'Game of Life' example from DANA, integrated into
Brian's framework.
'''

from brian import *
from scipy.signal import convolve2d
from pylab import gray

width = 50
N = width*width
steps = 64

duration = steps*defaultclock.dt

kernel = array([[1,1,1],
                [1,0,1],
                [1,1,1]])

G = NeuronGroup(N, 'alive:1')

G.alive = randint(2, size=N)
alive = reshape(G.alive, (width, width))

plotshape = int(ceil(sqrt(steps)))

plotnum = 1
@network_operation
def update():
    global plotnum
    neighbs = convolve2d(alive, kernel, mode='same')
    alive[:] = maximum(0, 1.0-(neighbs<1.5)-(neighbs>3.5)-(neighbs<2.5)*(1-alive))
    if plotnum<=plotshape*plotshape:
        subplot(plotshape, plotshape, plotnum)
        imshow(alive.copy(), interpolation='nearest')
        gray()
    plotnum += 1

run(10*ms)

show()



'''
Adapted (and simplified) from
Theory of Arachnid Prey Localization
W. Sturzl, R. Kempter, and J. L. van Hemmen
PRL 2000

The membrane potential is normalized
(i.e. 0 = rest, 1 = threshold).
'''
from brian import *

# Parameters
radius = 2.5 * cm
wave_speed = 50 * meter / second
prey_angle = 0
legs_angle = pi / 8 + pi / 4 * arange(8)

wave = lambda t:.2 * sin(2 * pi * 300 * Hz * t) * cos(2 * pi * 25 * Hz * t)

# Leg mechanical receptors
tau = 1 * ms
sigma = .01
eqs_legs = """
dv/dt=(1+wave(t-d)-v)/tau+sigma*(2./tau)**.5*xi:1
d : second # wave delay
"""
legs = NeuronGroup(8, model=eqs_legs, threshold=1, reset=0, refractory=1 * ms)
legs.d = radius / wave_speed * (1 - cos(prey_angle - legs_angle))

# Command neurons
eqs_neuron = '''
dv/dt=(x-v)/tau : 1
dx/dt=-x/tau : 1 # PSPs are alpha functions
'''
neurons = NeuronGroup(8, model=eqs_neuron, threshold=1, reset=0)
synapses_ex = IdentityConnection(legs, neurons, 'x', weight=4)
synapses_inh = Connection(legs, neurons, 'x', delay=.7 * ms)
for i in range(8):
    synapses_inh[i, (4 + i - 1) % 8] = -1
    synapses_inh[i, (4 + i) % 8] = -1
    synapses_inh[i, (4 + i + 1) % 8] = -1
spikes = SpikeCounter(neurons)

run(300 * ms)
polar(legs_angle, spikes.count)
print spikes.count
show()



speedtrack_desc = 'SpikeGeneratorGroup with a fixed array of spiketimes'

from brian import *
from tracking import *
import gc, time

# All neurons fire simultaneously with the given rate
# The spiketimes are either given as a list of (index, time) pairs or as an 
# array with first column index and second column time  

# Firing rates
rates = [1, 500, 1000, 2000, 5000] #Hz

# Spike times are given as pairs, as pairs but using gather=True, or as an array 
stim_types = ['pairs', 'array']

def run_barrage():
    results = []
    
    for stim_type in stim_types:
        for r in rates:
            print stim_type, str(r)
            t = TrackSpeed(r, stim_type)
            results.append(('%s (%dHz)' % (stim_type, r), track(t),
                            (r, stim_type)))
            gc.collect()
    return (speedtrack_desc, results)

def plot_results(name, results):
    for stim_type in stim_types:
        allt = [t for _, t, args in results  if args[1] == stim_type]
        alln = [args[0] for _, t, args in results if args[1] == stim_type]
        plot(alln, allt, 'o-', label=(stim_type))
    legend(frameon=False)
    xlabel('rate (Hz)')
    ylabel('time (s)')
    title(name)


class TrackSpeed(object):
             
    def __init__(self, rate, stim_type, duration=1): 
        
        N = 100
        dt = defaultclock.dt
        defaultclock.reinit()
        self.duration = duration
        
        #The spike times are shifted by half a dt to center the spikes in the bins
        times = arange(0, duration, 1./rate)             
        gather = False
        
        if stim_type == 'pairs':            
            spiketimes = [(idx, t*second +dt/2) for idx in xrange(N) for t in times]            
        elif stim_type == 'array':
            spiketimes = vstack([array([n, t]) for t in times for n in xrange(N)])          
        else:
            raise ValueError('Unknown stim_type (should be "pairs" or "array)')            
        
        start = time.time()
        G = SpikeGeneratorGroup(N, spiketimes, gather=gather)
        self.init_time = time.time() - start

        self.net = Network(G)
        self.net.prepare()

    def run(self):
        #simulate the initialization time by waiting...        
        time.sleep(self.init_time)        
        
        self.net.run(self.duration * second)

if __name__ == '__main__':
    name, results = run_barrage()
    print name
    for desc, t, args in results:
        print desc, ':', t
    plot_results(name, results)
    show()



from brian import *
from filterbank import Filterbank,FunctionFilterbank,ControlFilterbank,CombinedFilterbank,RestructureFilterbank
from filterbanklibrary import *
from linearfilterbank import *

__all__ = ['DCGC']

def set_parameters(cf,param):
    
    parameters=dict()
    parameters['b1'] = 1.81
    parameters['c1'] = -2.96
    parameters['b2'] = 2.17
    parameters['c2'] = 2.2
    parameters['decay_tcst'] = .5*ms
    parameters['lev_weight'] = .5
    parameters['level_ref'] = 50.
    parameters['level_pwr1'] = 1.5
    parameters['level_pwr2'] = .5
    parameters['RMStoSPL'] = 30.
    parameters['frat0'] = .2330
    parameters['frat1'] = .005
    parameters['lct_ERB'] = 1.5  #value of the shift in ERB frequencies
    parameters['frat_control'] = 1.08
    parameters['order_gc']=4
    parameters['ERBrate']= 21.4*log10(4.37*cf/1000+1)
    parameters['ERBwidth']= 24.7*(4.37*cf/1000 + 1)
    
    if param: 
        if not isinstance(param, dict): 
            raise Error('given parameters must be a dict')
        for key in param.keys():
            if not parameters.has_key(key):
                raise Exception(key + ' is invalid key entry for given parameters')
            parameters[key] = param[key]

    return parameters

#defition of the controler class
class AsymCompUpdate: 
    def __init__(self,target,samplerate,fp1,param):
        fp1=atleast_1d(fp1)
        self.iteration=0
        self.target=target
        self.samplerate=samplerate
        self.fp1=fp1             
        self.exp_deca_val = exp(-1/(param['decay_tcst'] *samplerate)*log(2))
        self.level_min = 10**(- param['RMStoSPL']/20)
        self.level_ref  = 10**(( param['level_ref'] - param['RMStoSPL'])/20)        
        self.b=param['b2']
        self.c=param['c2']
        self.lev_weight=param['lev_weight']
        self.level_ref=param['level_ref']
        self.level_pwr1=param['level_pwr1']
        self.level_pwr2=param['level_pwr2']
        self.RMStoSPL=param['RMStoSPL']
        self.frat0=param['frat0']
        self.frat1=param['frat1']
        self.level1_prev=-100
        self.level2_prev=-100
        self.p0=2
        self.p1=1.7818*(1-0.0791*self.b)*(1-0.1655*abs(self.c))
        self.p2=0.5689*(1-0.1620*self.b)*(1-0.0857*abs(self.c))
        self.p3=0.2523*(1-0.0244*self.b)*(1+0.0574*abs(self.c))
        self.p4=1.0724
    def __call__(self,*input):
         value1=input[0][-1,:]
         value2=input[1][-1,:]
         level1 = maximum(maximum(value1,0),self.level1_prev*self.exp_deca_val)
         level2 = maximum(maximum(value2,0),self.level2_prev*self.exp_deca_val)
         self.level1_prev=level1
         self.level2_prev=level2
         level_total=self.lev_weight*self.level_ref*(level1/self.level_ref)**self.level_pwr1+(1-self.lev_weight)*self.level_ref*(level2/self.level_ref)**self.level_pwr2
         level_dB=20*log10(maximum(level_total,self.level_min))+self.RMStoSPL                                       
         frat = self.frat0 + self.frat1*level_dB
         fr2 = self.fp1*frat
         self.iteration+=1
         self.target.filt_b, self.target.filt_a=asymmetric_compensation_coeffs(self.samplerate,fr2,self.target.filt_b,self.target.filt_a,self.b,self.c,self.p0,self.p1,self.p2,self.p3,self.p4)
 
class DCGC(CombinedFilterbank):
    '''
    The compressive gammachirp auditory filter as described in Irino, T. and
    Patterson R., "A compressive gammachirp auditory filter for both
    physiological and psychophysical data", JASA 2001.
    
    Technical implementation details and notation can be found in Irino, T. and
    Patterson R., "A Dynamic Compressive Gammachirp Auditory Filterbank",
    IEEE Trans Audio Speech Lang Processing.
    
    The model consists of a control pathway and a signal pathway in parallel.
    
    The control pathway consists of a bank of bandpass filters followed by a
    bank of highpass filters (this chain yields a bank of gammachirp filters).
    
    The signal pathway consist of a bank of fix bandpass filters followed by a
    bank of highpass filters with variable cutoff frequencies  (this chain
    yields a bank of gammachirp filters with a level-dependent bandwidth). The
    highpass filters of the signal pathway are controlled 
    by the output levels of the two stages of the control pathway. 
    
    Initialised with arguments:
    
    ``source``
        Source of the cochlear model.
        
    ``cf``
        List or array of center frequencies.
        
    ``update_interval``
        Interval in samples controlling how often the band pass filter of the
        signal pathway is updated. Smaller values are more accurate, but give
        longer computation times.
        
    ``param``
        Dictionary used to overwrite the default parameters given in the
        original paper.
    
    The possible parameters to change and their default values (see Irino, T.
    and Patterson R., "A Dynamic Compressive Gammachirp 
    Auditory Filterbank", IEEE Trans Audio Speech Lang Processing) are::
    
      param['b1'] = 1.81
      param['c1'] = -2.96
      param['b2'] = 2.17
      param['c2'] = 2.2
      param['decay_tcst'] = .5*ms
      param['lev_weight'] = .5
      param['level_ref'] = 50.
      param['level_pwr1'] = 1.5
      param['level_pwr2'] = .5
      param['RMStoSPL'] = 30.
      param['frat0'] = .2330
      param['frat1'] = .005
      param['lct_ERB'] = 1.5  #value of the shift in ERB frequencies
      param['frat_control'] = 1.08
      param['order_gc']=4
      param['ERBrate']= 21.4*log10(4.37*cf/1000+1) # cf is the center frequency
      param['ERBwidth']= 24.7*(4.37*cf/1000 + 1)
    '''
    
    def __init__(self, source,cf,update_interval,param={}):
        
        CombinedFilterbank.__init__(self, source)
        source = self.get_modified_source()
        
        parameters=set_parameters(cf,param)
        ERBspace = mean(diff(parameters['ERBrate']))
        cf=atleast_1d(cf)
        #bank of passive gammachirp filters. As the control path uses the same passive filterbank than the signal path (buth shifted in frequency)
        #this filterbanl is used by both pathway.
        pGc=LogGammachirp(source,cf,b=parameters['b1'], c=parameters['c1'])
#        self.gc.filt_b=pGc.filt_b
#        self.gc.filt_a=pGc.filt_a
        fp1 = cf + parameters['c1']*parameters['ERBwidth']*parameters['b1']/parameters['order_gc']
        nbr_cf=len(cf)
        #### Control Path ####
        n_ch_shift  = round(parameters['lct_ERB']/ERBspace); #value of the shift in channels
        indch1_control = minimum(maximum(1, arange(1,nbr_cf+1)+n_ch_shift),nbr_cf).astype(int)-1
        fp1_control = fp1[indch1_control]        
        pGc_control=RestructureFilterbank(pGc,indexmapping=indch1_control)
        fr2_control = parameters['frat_control']*fp1_control       
        asym_comp_control=AsymmetricCompensation(pGc_control, fr2_control,b=parameters['b2'], c=parameters['c2'])
        
        #### Signal Path ####
        fr1=fp1*parameters['frat0']
        signal_path= AsymmetricCompensation(pGc, fr1,b=parameters['b2'], c=parameters['c2'])
#        self.asym_comp.filt_b=signal_path.filt_b
#        self.asym_comp.filt_a=signal_path.filt_a
        #### Controler #### 
        updater = AsymCompUpdate(signal_path,source.samplerate,fp1,parameters)   #the updater
        control = ControlFilterbank(signal_path, [pGc_control,asym_comp_control], signal_path, updater, update_interval)  
        
        self.set_output(control)


'''
Tk interface
From http://www.pythonware.com/library/tkinter/introduction/hello-again.htm

Idea: extend the Parameters class to generate interfaces with load/save abilities
Maybe use configparser? (and take care of units)

Suggestion:
* use wxpython
* editors: wxglade, boa, pythoncard
'''
from Tkinter import *


class App:

    def __init__(self, master):

        frame = Frame(master)
        frame.pack()

        self.button = Button(frame, text="QUIT", fg="red", command=frame.quit)
        self.button.pack(side=LEFT)

        self.hi_there = Button(frame, text="Hello", command=self.say_hi)
        self.hi_there.pack(side=LEFT)

    def say_hi(self):
        print "hi there, everyone!"

root = Tk()

app = App(root)

root.mainloop()



from brian import *
from scipy.io import loadmat # NOTE: this requires scipy 0.7+
from glob import glob
from copy import copy
from scipy.io.wavfile import *
import os, re
from hrtf import *

__all__ = ['IRCAM_LISTEN']

class IRCAM_LISTEN(HRTFDatabase):
    '''
    :class:`HRTFDatabase` for the IRCAM LISTEN public HRTF database.
    
    For details on the database, see the
    `website <http://recherche.ircam.fr/equipes/salles/listen/>`__.
    
    The database object can be initialised with the following arguments:
    
    ``basedir``
        The directory where the database has been downloaded and extracted,
        e.g. ``r'D:\HRTF\IRCAM'``. Multiple directories in a list can be provided as well (e.g IRCAM and IRCAM New).
    ``compensated=False``
        Whether to use the raw or compensated impulse responses.
    ``samplerate=None``
        If specified, you can resample the impulse responses to a different
        samplerate, otherwise uses the default 44.1 kHz.
    
    The coordinates are pairs ``(azim, elev)`` where ``azim`` ranges from 0
    to 345 degrees in steps of 15 degrees, and elev ranges from -45 to 90 in
    steps of 15 degrees. After loading the database, the attribute 'subjects' gives all the subjects number that were detected as installed.
    
    **Obtaining the database**
    
    The database can be downloaded
    `here <http://recherche.ircam.fr/equipes/salles/listen/download.html>`__.
    Each subject archive should be extracted to a folder (e.g. IRCAM) with the
    names of the subject, e.g. IRCAM/IRC_1002, etc.
    '''
    def __init__(self, basedir, compensated=False, samplerate=None):
        if not isinstance(basedir, (list, tuple)):
            basedir = [basedir]
        self.basedir = basedir
        self.compensated = compensated
        names = []
        for basedir in self.basedir:
            names += glob(os.path.join(basedir, 'IRC_*'))
        splitnames = [os.path.split(name) for name in names]
        
        p = re.compile('IRC_\d{4,4}')
        self.subjects = [int(name[4:8]) for base, name in splitnames 
                         if not (p.match(name[-8:]) is None)]
        if samplerate is not None:
            raise ValueError('Custom samplerate not supported.')
        self.samplerate = samplerate

    def load_subject(self, subject, rounddot5 = False):


        subject = str(subject)
        if subject[0] == '3':
            # this is the case only for stuffed animals recordings
            # IRC_30..
            samplerate = 192*kHz
        else:
            samplerate = 44.1*kHz
        ok = False
        k = 0
        while k < len(self.basedir) and not ok:
            try:
                filename = os.path.join(self.basedir[k], 'IRC_' + subject)
                if self.compensated:
                    filename = os.path.join(filename, 'COMPENSATED/MAT/HRIR/IRC_' + subject + '_C_HRIR.mat')
                else:
                    filename = os.path.join(filename, 'RAW/MAT/HRIR/IRC_' + subject + '_R_HRIR.mat')
                m = loadmat(filename, struct_as_record=True)
                ok = True
            except IOError:
                ok = False
            k += 1
        if not ok:
            raise IOError("Couldn't find the HRTF files for subject "+str(subject))

        if 'l_hrir_S' in m.keys(): # RAW DATA
            affix = '_hrir_S'
        else:                      # COMPENSATED DATA
            affix = '_eq_hrir_S'
        l, r = m['l' + affix], m['r' + affix]

        azim = l['azim_v'][0][0][:, 0]
        elev = l['elev_v'][0][0][:, 0]
        if len(azim) == len(elev) and len(azim) == 1:
            # it is the case with IRCAM_New db
            # - the coordinates are 1xN instead of Nx1
            # - some measures that should be at the same elevation are
            # at very close but different elevations (7.47
            # vs. 7.5). This is annoying for interpolation. Hence I
            # allow one to round the elevations
            conv = lambda x : x
            if rounddot5:
                conv = lambda x: np.round(2*x)/2
            azim = conv(l['azim_v'][0][0][0, :])
            elev = l['elev_v'][0][0][0, :]
        coords = make_coordinates(azim=azim, elev=elev)
        l = l['content_m'][0][0]
        r = r['content_m'][0][0]
        # self.data has shape (num_ears=2, num_indices, hrir_length)
        data = vstack((reshape(l, (1,) + l.shape), reshape(r, (1,) + r.shape)))
        hrtfset = HRTFSet(data, samplerate, coords)
        hrtfset.name = 'IRCAM_'+subject
        return hrtfset



from brian import *
from customrefractoriness import *

#----Simulation parameters
cl = Clock(dt=0.1 * ms)
period = 5 * ms   #period of time over which the slope of depolarization is computed   
n = int(period / cl.dt)
N = 1
k = 16 * mV / ms

#----Fixed parameters
ENa = 60 * mV
El = -70 * mV
tau = 5 * ms
tauh = 5 * ms

#----Input parameters
mu = 5 * mV
sigma = 2 * 15 * mV
tauc = 10 * ms

#----Na channels parameters
va = -28.69 * mV
ka = 6.77 * mV
vi = -71.87 * mV
ki = 7.65 * mV
r = 10.8

eqs = Equations("""
# Sodium channel model
Pa=1/(1+exp(-(v-va)/ka)) : 1
Pi=1/(1+exp(-(v-vi)/ki)) : 1

# Membrane equation
dv/dt=(-r*Pa*h*(v-ENa)-(v-El)+I)/tau : volt
dh/dt=((1-Pi)-h)/tauh : 1

# Input
dI/dt=(mu-I)/tauc+sigma*(.5*tauc)**-.5*xi : volt
""")

def myreset(P, spikes):
    P.v[spikes] = El
    P.h[spikes] = 1

#neuron=NeuronGroup(N,model=eqs,threshold=0*mV,reset=myreset,refractory=50*ms)
neuron = NeuronGroup(N, model=eqs, threshold=0 * mV, reset=CustomRefractoriness(myreset, 50 * ms))

#----Initial conditions
neuron.v = El
neuron.h = 1

#----Record variables
Mv = StateMonitor(neuron, 'v', record=0)
Mh = StateMonitor(neuron, 'h', record=0)
Msp = SpikeMonitor(neuron)

run(150 * ms)

#--Fig2:Parameters time course---------------
figure()
subplot(211)
plot(Mv.times / ms, Mv[0] / mV)
subplot(212)
plot(Mh.times / ms, Mh[0])

show()



from brian.connections import SparseMatrix
from numpy import zeros, array
from nose.tools import *

def test_sparse_matrix():
    x = SparseMatrix((10, 10))
    z = zeros((10, 10), dtype=int)
    x[0, 4] = 3
    x[1, :] = 5
    x[2, []] = []
    x[3, [0]] = [6]
    x[4, [0, 1]] = [7, 8]
    x[5, 2:4] = [1, 2]
    x[6:8, 4:7] = 1
    # same thing with z to compare
    z[0, 4] = 3
    z[1, :] = 5
    z[2, []] = []
    z[3, [0]] = [6]
    z[4, [0, 1]] = [7, 8]
    z[5, 2:4] = [1, 2]
    z[6:8, 4:7] = 1
    # check the values are correct
    y = array(x.todense(), dtype=int)
    assert (y==z).all()

if __name__ == '__main__':
    test_sparse_matrix()



'''
Network of noisy IF neurons with gap junctions
'''
from brian import *

N = 300
v0 = 5 * mV
tau = 20 * ms
sigma = 5 * mV
vt = 10 * mV
vr = 0 * mV
g_gap = 1. / N
beta = 60 * mV * 2 * ms
delta = vt - vr

eqs = '''
dv/dt=(v0-v)/tau+g_gap*(u-N*v)/tau : volt
du/dt=(N*v0-u)/tau : volt # input from other neurons
'''

def myreset(P, spikes):
    P.v[spikes] = vr # reset
    P.v += g_gap * beta * len(spikes) # spike effect
    P.u -= delta * len(spikes)

group = NeuronGroup(N, model=eqs, threshold=vt, reset=myreset)

@network_operation
def noise(cl):
    x = randn(N) * sigma * (cl.dt / tau) ** .5
    group.v += x
    group.u += sum(x)

trace = StateMonitor(group, 'v', record=[0, 1])
spikes = SpikeMonitor(group)
rate = PopulationRateMonitor(group)

run(1 * second)
subplot(311)
raster_plot(spikes)
subplot(312)
plot(trace.times / ms, trace[0] / mV)
plot(trace.times / ms, trace[1] / mV)
subplot(313)
plot(rate.times / ms, rate.smooth_rate(5 * ms) / Hz)
show()



from brian import *
from brian.hears import*
from brian.hears import filtering
#filtering.use_gpu = False 

samplerate = 44.1*kHz
defaultclock.dt = 1/samplerate

cfmin, cfmax, cfN = 100*Hz, 2*kHz, 5
cf = erbspace(cfmin, cfmax, cfN)

sound = Sound(array([1]), rate=samplerate).extend(50*ms)

fb = GammatoneFilterbank(samplerate, cf)

G = FilterbankGroup(fb, sound)

M = StateMonitor(G, 'output', record=True)

run(sound.duration)

M.plot()
show()



'''
Script demonstrating use of a :class:`Connection` with homogenenous delays

The network consists of a 'starter' neuron which fires a single spike at time
t=0, connected to 100 leaky integrate and fire neurons with different delays
for each target neuron, with the delays forming a quadratic curve centred at
neuron 50. The longest delay is 10ms, and the network is run for 40ms. At
the end, the delays are plotted above a colour plot of the membrane potential
of each of the target neurons as a function of time (demonstrating the
delays).
'''
from brian import *
# Starter neuron, threshold is below 0 so it fires immediately, reset is below
# threshold so it fires only once.
G = NeuronGroup(1, model='V:1', threshold= -1.0, reset= -2.0)
# 100 LIF neurons, no reset or threshold so they will not spike
H = NeuronGroup(100, model='dV/dt=-V/(10*ms):volt')
# Connection with delays, here the delays are specified as a function of (i,j)
# giving the delay from neuron i to neuron j. In this case there is only one
# presynaptic neuron so i will be 0.
C = Connection(G, H, weight=5 * mV, max_delay=10 * ms,
               delay=lambda i, j:10 * ms * (j / 50. - 1) ** 2)
M = StateMonitor(H, 'V', record=True)
run(40 * ms)
subplot(211)
# These are the delays from neuron 0 to neuron i in ms
plot([C.delay[0, i] / ms for i in range(100)])
ylabel('Delay (ms)')
title('Delays')
subplot(212)
# M.values is an array of all the recorded values, here transposed to make
# it fit with the plot above.
imshow(M.values.T, aspect='auto', extent=(0, 100, 40, 0))
xlabel('Neuron number')
ylabel('Time (ms)')
title('Potential')
show()



'''
Very short example program.
'''
#import brian_no_units
from brian import *
from time import time

N = 10000        # number of neurons
Ne = int(N * 0.8) # excitatory neurons 
Ni = N - Ne       # inhibitory neurons
p = 80. / N
duration = 1000 * ms

eqs = '''
dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
dge/dt = -ge/(5*ms) : volt
dgi/dt = -gi/(10*ms) : volt
'''

P = NeuronGroup(N, model=eqs, threshold= -50 * mV, reset= -60 * mV)
P.v = -60 * mV + 10 * mV * rand(len(P))
Pe = P.subgroup(Ne)
Pi = P.subgroup(Ni)

Ce = Connection(Pe, P, 'ge', weight=1.62 * mV, sparseness=p)
Ci = Connection(Pi, P, 'gi', weight= -9 * mV, sparseness=p)

M = SpikeMonitor(P)
trace = StateMonitor(P, 'v', record=0)

t1 = time()
run(1 * second)
t2 = time()
print "Simulated in", t2 - t1, "s"
print len(M.spikes), "spikes"

subplot(211)
raster_plot(M)
subplot(212)
plot(trace.times / ms, trace[0] / mV)
show()



from codegen import *
from codegen_c import *
from rewriting import *

__all__ = ['GPUCodeGenerator']


class GPUCodeGenerator(CCodeGenerator):
    def initialisation(self, eqs):
        vartype = self.vartype()
        #code = '__global__ void stateupdate(int num_neurons, '+vartype+' t, '+vartype+' dt, '+vartype+' *S)\n'
        code = '__global__ void stateupdate(int num_neurons, ' + vartype + ' t, ' + vartype + ' dt, ' + ', '.join(vartype + ' *' + var + '_arr' for var in eqs._diffeq_names) + ')\n'
        code += '{\n'
        code += '    const int neuron_index = blockIdx.x * blockDim.x + threadIdx.x;\n'
        code += '    if(neuron_index>=num_neurons) return;\n'
#        for j, name in enumerate(eqs._diffeq_names):
#            code += '    '+vartype+' ' + name + ' = S[neuron_index+'+str(j)+'*num_neurons];\n'
        return code

    def finalisation(self, eqs):
        return '}\n'

    def generate(self, eqs, scheme):
        vartype = self.vartype()
        code = self.initialisation(eqs)
        for j, name in enumerate(eqs._diffeq_names):
            code += '    ' + vartype + ' ' + name + ' = ' + name + '_arr[neuron_index];\n'
        for line in self.scheme(eqs, scheme).split('\n'):
            line = line.strip()
            if line:
                if 'exp(' in line:
                    #code += '    '+line[:-1]+'*0.0+0.0000001;\n'
                    #code += '    '+line[:2]+'=0.0;\n'
                    line = line.replace('exp(', 'expf(')
                    code += '    ' + line + '\n'
                else:
                    code += '    ' + line + '\n'
        for j, name in enumerate(eqs._diffeq_names):
            if name in eqs._diffeq_names_nonzero:
                #code += 'if(isfinite('+name+')) '+name+'_arr[neuron_index] = '+name+';\n'
                code += '    ' + name + '_arr[neuron_index] = ' + name + ';\n'
                #code += '    '+name+'_arr[neuron_index] = (0.0);\n'
                #code += '    S[neuron_index+'+str(j)+'*num_neurons] = '+name+';\n'
        code += self.finalisation(eqs)
        return code



from brian import *
from modelfitting import *
import time, os

dt = defaultclock.dt

def compensate(current, trace, popsize = 100, maxiter = 10,
               equations = None, reset = None, threshold = None,
                  slice_duration = 1 * second, overlap = 100*ms,
                  initial = None, dt = defaultclock.dt,
                  cpu = None, gpu = 1, record=['V','Ve'],
                  cut = None, cut_length = 0.0,
                  p = 0.5, best_params=None,
                  **params):
    
    trace0 = trace.copy()
    
    if initial is None: 
        initial = trace[0]
    
    if slice_duration is None:
        slices = 1
    else:
        slices = max(int(len(current)*dt/slice_duration), 1)
    current0 = current
    current, trace = slice_trace(current, trace, slices=slices, overlap=overlap, dt=dt)
    
    if cut is not None:
        cut, cut_indices = transform_spikes(popsize, current0, [(0, float(c)) for c in cut], 
                                            slices=slices, overlap=overlap, dt=dt)
        cut_steps = int32(cut_length/dt)
        cut = array(cut/dt, dtype=int32)
        cut_indices = array(cut_indices, dtype=int32)
    else:
        cut_indices = None
        cut_steps = 0
    
    initial_values = {'V0': initial, 'Ve': 0.}
    
    if equations is None:
        equations = Equations('''
                dV0/dt=(R*Iinj-V0+Vr)/tau : 1
                Iinj=(V-V0)/Re : 1
                dV/dt=Re*(I-Iinj)/taue : 1
                Ve=V-V0 : 1
                I : 1
                R : 1
                Re : 1
                Vr : 1
                tau : second
                taue : second
            ''')
    if threshold is None:
        threshold = "V>100000"
    if reset is None:
        reset = ""
    
    if len(params) == 0:
        params = dict(R =  [1.0e3, 1.0e6, 1000.0e6, 1.0e12],
                      Re = [1.0e3, 1.0e6, 1000.0e6, 1.0e12],
                      Vr = [-100.0e-3, -80e-3, -40e-3, -10.0e-3],
                      tau =  [.1*ms, 1*ms, 30*ms, 200*ms],
                      taue = [.01*ms, .1*ms, 5*ms, 20*ms])
    
    criterion = LpError(p=p, varname='V')
    
    if best_params is None:
        results = modelfitting( model = equations,
                                reset = reset,
                                threshold = threshold,
                                data = trace,
                                input = current,
                                cpu = cpu,
                                gpu = gpu,
                                dt = dt,
                                popsize = popsize,
                                maxiter = maxiter,
                                onset = overlap,
                                criterion = criterion,
                                initial_values = initial_values,
                                **params
                                )
        print_table(results)
        best_params = results.best_params
    else:
        results = best_params
        for key in best_params.keys():
            best_params[key] = [best_params[key]]
    
    criterion_values, record_values = simulate( model = equations,
                                                reset = reset,
                                                threshold = threshold,
                                                data = trace,
                                                input = current,
                                                dt = dt,
                                                neurons = slices, # 1 neuron/slice
                                                onset = overlap,
                                                criterion = criterion,
                                                use_gpu = True,
                                                record = record,
                                                initial_values = initial_values,
                                                **best_params
                                                )
    
    traceV = record_values[0][:,int(overlap/dt):].flatten()
    traceVe = record_values[1][:,int(overlap/dt):].flatten()
    
    
#    return traceV, traceVe, results
    return trace0 - traceVe, traceV, traceVe, results



'''
Very short example program.
'''
from brian import *
import Pyro.core
from Pyro.errors import PyroError


class BrianServer(Pyro.core.ObjBase):
    def __init__(self):
        Pyro.core.ObjBase.__init__(self)

Pyro.core.initServer()
daemon = Pyro.core.Daemon()
print
print 'The Pyro Daemon is running on ', daemon.hostname + ':' + str(daemon.port)
print '(you may need this info for the client to connect to)'
print

eqs = '''
dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
dge/dt = -ge/(5*ms) : volt
dgi/dt = -gi/(10*ms) : volt
'''

P = NeuronGroup(4000, model=eqs,
              threshold= -50 * mV, reset= -60 * mV)
P.v = -60 * mV + 10 * mV * rand(len(P))
Pe = P.subgroup(3200)
Pi = P.subgroup(800)

Ce = Connection(Pe, P, 'ge', weight=1.62 * mV, sparseness=0.02)
Ci = Connection(Pi, P, 'gi', weight= -9 * mV, sparseness=0.02)

M = SpikeMonitor(P)

# Pyro loop
@network_operation
def server():
    pass

run(1 * second)
raster_plot(M)
show()



'''
Apply style changes to all Brian modules.

From PEP-8 (Python naming conventions):
---------------------------------------
Package and Module Names

      Modules should have short, all-lowercase names.  Underscores can be used
      in the module name if it improves readability.  Python packages should
      also have short, all-lowercase names, although the use of underscores is
      discouraged.

      Since module names are mapped to file names, and some file systems are
      case insensitive and truncate long names, it is important that module
      names be chosen to be fairly short -- this won't be a problem on Unix,
      but it may be a problem when the code is transported to older Mac or
      Windows versions, or DOS.

      When an extension module written in C or C++ has an accompanying Python
      module that provides a higher level (e.g. more object oriented)
      interface, the C/C++ module has a leading underscore (e.g. _socket).

    Class Names

      Almost without exception, class names use the CapWords convention.
      Classes for internal use have a leading underscore in addition.

    Exception Names

      Because exceptions should be classes, the class naming convention
      applies here.  However, you should use the suffix "Error" on your
      exception names (if the exception actually is an error).

    Global Variable Names

      (Let's hope that these variables are meant for use inside one module
      only.)  The conventions are about the same as those for functions.

      Modules that are designed for use via "from M import *" should use the
      __all__ mechanism to prevent exporting globals, or use the older
      convention of prefixing such globals with an underscore (which you might
      want to do to indicate these globals are "module non-public").

    Function Names

      Function names should be lowercase, with words separated by underscores
      as necessary to improve readability.

      mixed_case is allowed only in contexts where that's already the
      prevailing style (e.g. threading.py), to retain backwards compatibility.

    Function and method arguments

      Always use 'self' for the first argument to instance methods.

      Always use 'cls' for the first argument to class methods.

      If a function argument's name clashes with a reserved keyword, it is
      generally better to append a single trailing underscore rather than use
      an abbreviation or spelling corruption.  Thus "print_" is better than
      "prnt".  (Perhaps better is to avoid such clashes by using a synonym.)

    Method Names and Instance Variables

      Use the function naming rules: lowercase with words separated by
      underscores as necessary to improve readability.

      Use one leading underscore only for non-public methods and instance
      variables.

      To avoid name clashes with subclasses, use two leading underscores to
      invoke Python's name mangling rules.

      Python mangles these names with the class name: if class Foo has an
      attribute named __a, it cannot be accessed by Foo.__a.  (An insistent
      user could still gain access by calling Foo._Foo__a.)  Generally, double
      leading underscores should be used only to avoid name conflicts with
      attributes in classes designed to be subclassed.

      Note: there is some controversy about the use of __names (see below).
'''

import os, re

os.chdir('..')
os.chdir('..')
os.chdir('..') # work from Brian's root

#package_names={'Brian':'brian'}
#module_names={'BrianNoUnits':'brian_no_units'}
specific_changes = {'is_number_type':'isNumberType',
                  'is_sequence_type':'isSequenceType',
                  'assert_raises':'assertRaises',
                  'BrianNoUnits':'brian_no_units',
                  'BrianNoUnitsNoWarnings':'brian_no_units_no_warnings',
                  'BrianUnitPrefs':'brian_unit_prefs',
                  'from brian':'from brian',
                  'Library':'library'}

def bad_identifiers(text):
    return set(re.findall(r'\b[a-z0-9_]+[A-Z]\w+\b', text))

def replace_bad_identifiers(text, names):
    text2 = text
    for name in names:
        new_name = re.sub('[A-Z]', lambda c:'_' + c.group(0).lower(), name)
        text2 = re.sub(name, new_name, text2)
    return text2

def make_specific_changes(text):
    text2 = text
    for name1, name2 in specific_changes.iteritems():
        text2 = re.sub(name1, name2, text2)
    return text2

def patch_file(filename):
    f_in = open(filename)
    text = f_in.read()
    f_in.close()

    names = bad_identifiers(text)
    text_out = text
    if names != set([]):
        text_out = replace_bad_identifiers(text_out, names)
    text_out = make_specific_changes(text_out)

    if text != text_out:
        f_out = open(filename, 'w')
        f_out.write(text_out)
        f_out.close()

def patch_all_files(verbose=True):
    # Get all source files
    files = []
    for dirpath, dirnames, filenames in os.walk('.'):
        files.extend([dirpath + '\\' + file for file in filenames if file[-3:] == '.py'])
    for file in files:
        if file[-13:] != 'stylepatch.py':
            if verbose:
                print file
            patch_file(file)

if __name__ == '__main__':
    patch_all_files()



from brian import *
set_global_preferences(usenewbrianhears=True)
from brian.hears import *

x1 = Sound((whitenoise(100*ms), whitenoise(100*ms)))
x1.level = (0*dB, 10*dB)
x2 = Sound((whitenoise(100*ms), whitenoise(100*ms)))
x2.level = (20*dB, 30*dB)
x3 = Sound((whitenoise(100*ms), whitenoise(100*ms)))
x3.level = (40*dB, 50*dB)
x4 = Sound((whitenoise(100*ms), whitenoise(100*ms)))
x4.level = (60*dB, 70*dB)
x5 = Sound((whitenoise(100*ms), whitenoise(100*ms)))
x5.level = (80*dB, 90*dB)

fb1 = RestructureFilterbank((x1, x2), numrepeat=3, type='serial')
fb2 = RestructureFilterbank((x3, x4), type='interleave', numtile=3)
fb3 = RestructureFilterbank((fb1, fb2), type='interleave')
rep = Repeat(x1, 3)
fb4 = Interleave(Join(rep,
                      Repeat(x2, 3)),
                 Tile(Interleave(x3, x4), 3))

print fb1.indexmapping
print fb2.indexmapping
print fb3.indexmapping
print fb4.indexmapping

names = ['x1', 'x2', 'x3', 'x4', 'x5', 'fb1', 'fb2', 'fb3', 'fb4']
objids = [eval('id('+name+')') for name in names]
namefromid = dict(zip(objids, names))

for s in fb3.source:
    print namefromid[id(s)]
print '---'
for s in fb4.source:
    print namefromid[id(s)]
    
y = Sound(fb3.buffer_fetch(0, len(x1)))

print y.level

rep.source = x5
fb4.buffer_init()
print Sound(fb4.buffer_fetch(0, len(x1))).level

for s in fb4.source:
    print namefromid[id(s)]



from sympy.printing.ccode import CCodePrinter
from sympy.printing.str import StrPrinter
from sympy.printing.precedence import precedence, PRECEDENCE
from sympy.core.basic import S

def boolean_printer(origop, newop, start=''):
    def f(self, expr):
        PREC = precedence(expr)
        return start + newop.join(self.parenthesize(a, PREC) for a in expr.args)
    f.__name__ = '_print_' + origop
    return f


class NewCCodePrinter(CCodePrinter):
    _print_And = boolean_printer('And', '&&')
    _print_Or = boolean_printer('Or', '||')
    _print_Not = boolean_printer('Not', '', '!')

def newccode(expr):
    return NewCCodePrinter().doprint(expr)

if __name__ == '__main__':
    from sympy import *

    x = Symbol('x')
    y = Symbol('y')

    z = ~(x + y)

    print newccode(z)



from brian import *
from brian.experimental.codegen.codegen_c import *
from brian.experimental.codegen.integration_schemes import *
import time
from scipy import weave
import numpy, scipy
from brian.experimental.codegen.c_support_code import *
from handythread import *
import re


class MulticoreCStateUpdater(StateUpdater):
    def __init__(self, eqs, scheme, clock=None, freeze=False, threads=1):
        self.threads = threads
        self.clock = guess_clock(clock)
        self.code_c = CCodeGenerator().generate(eqs, scheme)
        self.code_c = "Py_BEGIN_ALLOW_THREADS\n" + self.code_c + "Py_END_ALLOW_THREADS\n"
        code = ''
        for line in self.code_c.split('\n'):
            if line.startswith('double *') and line.endswith('num_neurons;'):
                line = line[:-1] + '+offset;'
            if line == 'for(int _i=0;_i<num_neurons;_i++){':
                line = 'for(int _i=0; _i<num_neurons_segment; _i++){'
            code += line + '\n'
        self.code_c = code
        print self.code_c
        log_debug('brian.experimental.codegen.stateupdaters', 'C state updater code:\n' + self.code_c)
        self._weave_compiler = get_global_preference('weavecompiler')
        self._extra_compile_args = ['-O3']
        if self._weave_compiler == 'gcc':
            self._extra_compile_args += ['-march=native', '-ffast-math']
        self._prepared = False

    def __call__(self, P):
        if not self._prepared:
            dt = P.clock._dt
            num_neurons = len(P)
            num_neurons_segment = num_neurons
            offset = 0
            _S = P._S
            t = P.clock._t
            weave.inline(self.code_c, ['_S', 'num_neurons', 'num_neurons_segment',
                                       'dt', 't', 'offset'],
                         support_code=c_support_code,
                         compiler=self._weave_compiler,
                         extra_compile_args=self._extra_compile_args,
                         #force=True,
                         )
            def makefunc(_S, offset, num_neurons, num_neurons_segment, dt):
                d = {'_S':_S, 'offset':int(offset), 'num_neurons':int(num_neurons),
                     'num_neurons_segment':int(num_neurons_segment), 'dt':dt}
                def f(t):
                    d['t'] = t
                    weave.inline(self.code_c, ['_S', 'num_neurons', 'num_neurons_segment',
                                               'dt', 't', 'offset'],
                                 local_dict=d,
                                 support_code=c_support_code,
                                 compiler=self._weave_compiler,
                                 extra_compile_args=self._extra_compile_args)
                return f
            pieces = array([len(P) / self.threads for _ in range(self.threads)], dtype=int)
            pieces[-1] += len(P) - sum(pieces)
            offsets = hstack((0, cumsum(pieces)))[:-1]
            self.funcs = [makefunc(_S, offset, num_neurons, piece, dt) for offset, piece in zip(offsets, pieces)]
            self._prepared = True
        else:
            t = P.clock._t
            parallel_map(lambda f:f(t), self.funcs, threads=self.threads)

if __name__ == '__main__':
    from brian.library.ionic_currents import *

    N = 10000
    duration = 100 * ms
    use_multicore = True
    numthreads = 2
    record_and_plot = False

    El = 10.6 * mV
    EK = -12 * mV
    ENa = 120 * mV
    eqs = MembraneEquation(1 * uF) + leak_current(.3 * msiemens, El)
    eqs += K_current_HH(36 * msiemens, EK) + Na_current_HH(120 * msiemens, ENa)
    eqs += Current('I:amp')
    eqs.prepare()

    neuron = NeuronGroup(N, eqs, implicit=True, freeze=True)

    if use_multicore:
        neuron._state_updater = MulticoreCStateUpdater(eqs, exp_euler_scheme,
                                                       threads=numthreads)

    if record_and_plot:
        trace = StateMonitor(neuron, 'vm', record=True)

    neuron.I = 10 * uA

    net = MagicNetwork()
    net.run(1 * ms)

    start = time.time()
    net.run(duration)
    end = time.time()

    print end - start

    if record_and_plot:
        trace.plot()
        show()



# From Ariel Zylberberg
from brian import *

vl = -70 * mvolt
vth = -50 * mvolt
vreset = -55 * mvolt
tauAMPA = 2 * ms
tauNMDA = 100 * ms
tauGABA = 10 * ms
a = 0.062 * mvolt ** -1
b = 3.57 * 1
Mg = 1
ve = 0 * mvolt
vi = -70 * mvolt

eqs = '''
    dv/dt = (-gl*(v-vl)-Isyn)*(1./cm) : volt
    Isyn  = IAMPA+INMDA+IGABA : amp
    IAMPA = (gextAMPA*sextAMPA+grecAMPA*srecAMPA)*(v-ve) : amp
    INMDA = gNMDA*sNMDA*(v-ve)/(1+Mg*exp(-a*v)/b) : amp
    IGABA = gGABA*sGABA*(v-vi) : amp
    dsextAMPA/dt = -sextAMPA/tauAMPA : 1
    dsrecAMPA/dt = -srecAMPA/tauAMPA : 1
    dsNMDA/dt    = -sNMDA/tauNMDA : 1 
    dsGABA/dt    = -sGABA/tauGABA : 1
    dss/dt = (0.63-ss)/tauNMDA : 1
    cm : nfarad
    gl : nsiemens
    gextAMPA : nsiemens
    grecAMPA : nsiemens
    gNMDA : nsiemens
    gGABA : nsiemens
    '''

N = 1000
Ne = 800
Ni = 200

Ge = NeuronGroup(Ne, model=eqs, threshold=vth, reset=vreset, refractory=2 * msecond)
Gi = NeuronGroup(Ni, model=eqs, threshold=vth, reset=vreset, refractory=1 * msecond)

Ge.cm = 0.5 * nfarad
Ge.gl = 25 * nsiemens
Ge.gextAMPA = 2.08 * nsiemens
Ge.grecAMPA = 0.104 * nsiemens
Ge.gNMDA = 0.327 * nsiemens
Ge.gGABA = 1.25 * nsiemens

Gi.cm = 0.2 * nfarad
Gi.gl = 20 * nsiemens
Gi.gextAMPA = 1.62 * nsiemens
Gi.grecAMPA = 0.081 * nsiemens
Gi.gNMDA = 0.258 * nsiemens
Gi.gGABA = 0.973 * nsiemens

#init
Ge.ss = 0
Ge.sNMDA = 0
Ge.sGABA = 0
Ge.sextAMPA = 0
Ge.srecAMPA = 0
Ge.v = -60 * mV

Gi.ss = 0
Gi.sNMDA = 0
Gi.sGABA = 0
Gi.sextAMPA = 0
Gi.srecAMPA = 0
Gi.v = -60 * mV

#recurrent connections
def reset_fun(P, spikes):
    P.v_[spikes] = vreset
    P.ss_[spikes] = 0

C = []
init_list = [(Ge, Ge, 'srecAMPA', None),
           (Ge, Gi, 'srecAMPA', None),
           (Ge, Ge, 'sNMDA', 'ss')]
for source, target, variable, modulation in init_list:
    C.append(Connection(source, target, variable, modulation=modulation))
    C[-1].connect_full(source, target, weight=1)

C1 = Connection(Ge, Ge, 'srecAMPA')
C1.connect_full(Ge, Ge, weight=1)

C2 = Connection(Ge, Gi, 'srecAMPA')
C2.connect_full(Ge, Gi, weight=1)

C3 = Connection(Ge, Ge, 'sNMDA', modulation='ss')
C3.connect_full(Ge, Ge, weight=1)

C4 = Connection(Ge, Gi, 'sNMDA', modulation='ss')
C4.connect_full(Ge, Gi, weight=1)

C5 = Connection(Gi, Ge, 'sGABA')
C5.connect_full(Gi, Ge, weight=1)

C6 = Connection(Gi, Gi, 'sGABA')
C6.connect_full(Gi, Gi, weight=1)

#external connections
Pe = PoissonGroup(Ne, 2400 * Hz)
Ce = Connection(Pe, Ge, 'sextAMPA')
Ce.connect_one_to_one(Pe, Ge, weight=1)

Pi = PoissonGroup(Ni, 2400 * Hz)
Ci = Connection(Pi, Gi, 'sextAMPA')
Ci.connect_one_to_one(Pi, Gi, weight=1)

#record
Me = PopulationRateMonitor(Ge, bin=25 * ms)
Mi = PopulationRateMonitor(Gi, bin=25 * ms)

#run
run(10.0 * second)

#plot
plot(Me.times / ms, Me.rate / Hz)
plot(Mi.times / ms, Mi.rate / Hz)
show()





from brian import *
nbr_neurons=2

gl=2*nS*linspace(0, 1, nbr_neurons)
El=-65*mV*ones(nbr_neurons)
#C=12*pF*ones(nbr_neurons) ## that was changed from previous example
C=12*pF
eqs_leak="""
ileak = gl*(El-v) : amp
"""
eqs="""
dv/dt=(ileak)/C : volt
"""
eqs+=eqs_leak
neuron=NeuronGroup(nbr_neurons,model=eqs, compile=True)#method='Euler')
M = StateMonitor(neuron, 'v', record=True)
run(100*ms,report='text')
M.plot()
show()



from brian import *

__all__ = ['SimpleCustomRefractoriness', 'CustomRefractoriness']


class SimpleCustomRefractoriness(Refractoriness):
    '''
    Holds the state variable at the custom reset value for a fixed time after a spike.
    
    **Initialised as:** ::
    
        SimpleCustomRefractoriness(resetfunc[,period=5*ms[,state=0]])
    
    with arguments:
    
    ``resetfun``
        The custom reset function ``resetfun(P, spikes)`` for ``P`` a
        :class:`NeuronGroup` and ``spikes`` a list of neurons that
        fired spikes.
    ``period``
        The length of time to hold at the reset value.
    ``state``
        The name or number of the state variable to reset and hold,
        it is your responsibility to check that this corresponds to
        the custom reset function.
    
    The assumption is that ``resetfun(P, spikes)`` will reset the state
    variable ``state`` on the group ``P`` for the spikes with indices
    ``spikes``. The values assigned by the custom reset function are
    stored by this object, and they are clamped at these values for
    ``period``. This object does not introduce refractoriness for more
    than the one specified variable ``state`` or for spike indices
    other than those in the variable ``spikes`` passed to the custom
    reset function.
    '''

    @check_units(period=second)
    def __init__(self, resetfun, period=5 * msecond, state=0):
        self.period = period
        self.resetfun = resetfun
        self.state = state
        self._periods = {} # a dictionary mapping group IDs to periods
        self.statevectors = {}
        self.lastresetvalues = {}

    def __call__(self, P):
        '''
        Clamps state variable at reset value.
        '''
        # if we haven't computed the integer period for this group yet.
        # do so now
        if id(P) in self._periods:
            period = self._periods[id(P)]
        else:
            period = int(self.period / P.clock.dt) + 1
            self._periods[id(P)] = period
        V = self.statevectors.get(id(P), None)
        if V is None:
            V = P.state_(self.state)
            self.statevectors[id(P)] = V
        LRV = self.lastresetvalues.get(id(P), None)
        if LRV is None:
            LRV = zeros(len(V))
            self.lastresetvalues[id(P)] = LRV
        lastspikes = P.LS.lastspikes()
        self.resetfun(P, lastspikes)             # call custom reset function 
        LRV[lastspikes] = V[lastspikes]         # store a copy of the custom resetted values
        clampedindices = P.LS[0:period]
        V[clampedindices] = LRV[clampedindices] # clamp at custom resetted values

    def __repr__(self):
        return 'Custom refractory period, ' + str(self.period)


class CustomRefractoriness(Refractoriness):
    '''
    Holds the state variable at the custom reset value for a fixed time after a spike.
    
    **Initialised as:** ::
    
        CustomRefractoriness(resetfunc[,period=5*ms[,refracfunc=resetfunc]])
    
    with arguments:
    
    ``resetfunc``
        The custom reset function ``resetfunc(P, spikes)`` for ``P`` a
        :class:`NeuronGroup` and ``spikes`` a list of neurons that
        fired spikes.
    ``refracfunc``
        The custom refractoriness function ``refracfunc(P, indices)`` for ``P`` a
        :class:`NeuronGroup` and ``indices`` a list of neurons that are in
        their refractory periods. In some cases, you can choose not to specify this,
        and it will use the reset function.
    ``period``
        The length of time to hold at the reset value.    
    '''

    @check_units(period=second)
    def __init__(self, resetfun, period=5 * msecond, refracfunc=None):
        self.period = period
        self.resetfun = resetfun
        if refracfunc is None:
            refracfunc = resetfun
        self.refracfunc = refracfunc
        self._periods = {} # a dictionary mapping group IDs to periods

    def __call__(self, P):
        '''
        Clamps state variable at reset value.
        '''
        # if we haven't computed the integer period for this group yet.
        # do so now
        if id(P) in self._periods:
            period = self._periods[id(P)]
        else:
            period = int(self.period / P.clock.dt) + 1
            self._periods[id(P)] = period
        lastspikes = P.LS.lastspikes()
        self.resetfun(P, lastspikes)             # call custom reset function
        clampedindices = P.LS[0:period]
        self.refracfunc(P, clampedindices)

    def __repr__(self):
        return 'Custom refractory period, ' + str(self.period)

if __name__ == '__main__':
    def f(P, spikes):
        P.V[spikes] = rand(len(spikes)) * 0.5
    R = SimpleCustomRefractoriness(f)
    G = NeuronGroup(5,
            model='''
            dV/dt = -(V-1.1)/(5*ms) : 1
            ''', reset=R, threshold=1)
    M = StateMonitor(G, 'V', record=True)
    run(1 * second)
    for i in range(5):
        plot(M.times, M[i])
    show()



from brian import *
from nose.tools import *
from brian.utils.approximatecomparisons import is_approx_equal

def test():
    '''
    :class:`Connection`
    ~~~~~~~~~~~~~~~~~~~
    
    **Initialised as:** ::
    
        Connection(source, target[, state=0[, delay=0*ms]])
    
    With arguments:
    
    ``source``
        The group from which spikes will be propagated.
    ``target``
        The group to which spikes will be propagated.
    ``state``
        The state variable name or number that spikes will be
        propagated to in the target group.
    ``delay``
        The delay between a spike being generated at the source
        and received at the target. At the moment, the mechanism
        for delays only works for relatively short delays (an
        error will be generated for delays that are too long), but
        this is subject to change. The exact behaviour then is
        not part of the assured interface, although it is very
        likely that the syntax will not change (or will at least
        be backwards compatible).
    
    **Methods**
    
    ``connect_random(P,Q,p[,weight=1])``
        Connects each neuron in ``P`` to each neuron in ``Q``.
    ``connect_full(P,Q[,weight=1])``
        Connect every neuron in ``P`` to every neuron in ``Q``.
    ``connect_one_to_one(P,Q)``
        If ``P`` and ``Q`` have the same number of neurons then neuron ``i``
        in ``P`` will be connected to neuron ``i`` in ``Q`` with weight 1.
    
    Additionally, you can directly access the matrix of weights by writing::
    
        C = Connection(P,Q)
        print C[i,j]
        C[i,j] = ...
    
    Where here ``i`` is the source neuron and ``j`` is the target neuron.
    Note: No unit checking is currently done if you use this method,
    but this is subject to change for future releases.

    The behaviour when a list of neuron ``spikes`` is received is to
    add ``W[i,:]`` to the target state variable for each ``i`` in ``spikes``. 
    '''
    reinit_default_clock()

    # test Connection object

    eqs = '''
    da/dt = 0.*hertz : 1.
    db/dt = 0.*hertz : 1.
    '''

    spikes = [(0, 1 * msecond), (1, 3 * msecond)]

    G1 = SpikeGeneratorGroup(2, spikes)
    G2 = NeuronGroup(2, model=eqs, threshold=10., reset=0.)

    # first test the methods
    # connect_full
    C = Connection(G1, G2)
    C.connect_full(G1, G2, weight=2.)
    for i in range(2):
        for j in range(2):
            assert (is_approx_equal(C[i, j], 2.))
    # connect_random
    C = Connection(G1, G2)
    C.connect_random(G1, G2, 0.5, weight=2.)
    # can't assert anything about that
    # connect_one_to_one
    C = Connection(G1, G2)
    C.connect_one_to_one(G1, G2)
    for i in range(2):
        for j in range(2):
            if i == j:
                assert (is_approx_equal(C[i, j], 1.))
            else:
                assert (is_approx_equal(C[i, j], 0.))
    del C
    # and we will use a specific set of connections in the next part
    Ca = Connection(G1, G2, 'a')
    Cb = Connection(G1, G2, 'b')
    Ca[0, 0] = 1.
    Ca[0, 1] = 1.
    Ca[1, 0] = 1.
    #Ca[1,1]=0 by default
    #Cb[0,0]=0 by default
    Cb[0, 1] = 1.
    Cb[1, 0] = 1.
    Cb[1, 1] = 1.
    net = Network(G1, G2, Ca, Cb)
    net.run(2 * msecond)
    # after 2 ms, neuron 0 will have fired, so a 0 and 1 should
    # have increased by 1 to [1,1], and b 1 should have increased
    # by 1 to 1
    assert (is_approx_equal(G2.a[0], 1.))
    assert (is_approx_equal(G2.a[1], 1.))
    assert (is_approx_equal(G2.b[0], 0.))
    assert (is_approx_equal(G2.b[1], 1.))
    net.run(2 * msecond)
    # after 4 ms, neuron 1 will have fired, so a 0 should have
    # increased by 1 to 2, and b 0 and 1 should have increased
    # by 1 to [1, 2]
    assert (is_approx_equal(G2.a[0], 2.))
    assert (is_approx_equal(G2.a[1], 1.))
    assert (is_approx_equal(G2.b[0], 1.))
    assert (is_approx_equal(G2.b[1], 2.))

    reinit_default_clock()

if __name__ == '__main__':
    test()



from brian import *
from nose.tools import *
from brian.utils.approximatecomparisons import is_approx_equal

def test():
    """
    :class:`Reset`
    ~~~~~~~~~~~~~~
    
    Initialised as::
    
        R = Reset(resetvalue=0*mvolt, state=0)

    After a neuron from a group with this reset fires, it
    will set the specified state variable to the given value.
    State variable 0 is customarily the membrane voltage,
    but this isn't required. 
    
    :class:`FunReset`
    ~~~~~~~~~~~~~~~~~
    
    Initialised as::
    
        R = FunReset(resetfun)
    
    Where resetfun is a function taking two arguments, the group
    it is acting on, and the indices of the spikes to be reset.
    The following is an example reset function::
    
        def f(P,spikeindices):
            P._S[0,spikeindices]=array([i/10. for i in range(len(spikeindices))])    
    
    :class:`Refractoriness`
    ~~~~~~~~~~~~~~~~~~~~~~~
    
    Initialised as::
    
        R = Refractoriness(resetvalue=0*mvolt,period=5*msecond,state=0)
    
    After a neuron from a group with this reset fires, the specified state
    variable of the neuron will be set to the specified resetvalue for the
    specified period.
    
    :class:`NoReset`
    ~~~~~~~~~~~~~~~~
    
    Initialised as::
    
        R = NoReset()
        
    Does nothing.
    """
    reinit_default_clock()


    # test that reset works as expected
    # the setup below is that group G starts with state values (1,1,1,1,1,0,0,0,0,0) threshold
    # value 0.5 (which should be initiated for the first 5 neurons) and reset 0.2 so that the
    # final state should be (0.2,0.2,0.2,0.2,0.2,0,0,0,0,0) 
    G = NeuronGroup(10, model=LazyStateUpdater(), reset=Reset(0.2), threshold=Threshold(0.5), init=(0.,))
    G1 = G.subgroup(5)
    G2 = G.subgroup(5)
    G1.state(0)[:] = array([1.] * 5)
    G2.state(0)[:] = array([0.] * 5)
    net = Network(G)
    net.run(1 * msecond)
    assert (all(G1.state(0) < 0.21) and all(0.19 < G1.state(0)) and all(G2.state(0) < 0.01))

    # check that function reset works as expected
    def f(P, spikeindices):
        P._S[0, spikeindices] = array([i / 10. for i in range(len(spikeindices))])
        P.called_f = True
    G = NeuronGroup(10, model=LazyStateUpdater(), reset=FunReset(f), threshold=Threshold(2.), init=(3.,))
    G.called_f = False
    net = Network(G)
    net.run(1 * msecond)
    assert (G.called_f)
    for i, v in enumerate(G.state(0)):
        assert (is_approx_equal(i / 10., v))

    # check that refractoriness works as expected
    # the network below should start at V=15, immediately spike as it is above threshold=1,
    # then should be clamped at V=-.5 until t=1ms at which point it should quickly evolve
    # via the DE to a value near 0 (and certainly between -.5 and 0). We test that the
    # value at t=0.5 is exactly -.5 and the value at t=1.5 is between -0.4 and 0.1 (to
    # avoid floating point problems)
    dV = 'dV/dt=-V/(.1*msecond):1.'
    G = NeuronGroup(1, model=dV, threshold=1., reset=Refractoriness(-.5, 1 * msecond))
    G.V = 15.
    net = Network(G)
    net.run(0.5 * msecond)
    for v in G.state(0):
        assert (is_approx_equal(v, -.5))
    net.run(1 * msecond)
    for v in G.state(0):
        assert (-0.4 < v < 0.1)

    get_default_clock().reinit()

if __name__ == '__main__':
    test()



'''
Example of the use of the function spike_triggered_average. A white noise  is filtered
by a gaussian filter (low pass filter) which output is used to generate spikes (poission process)
Those spikes are used in conjunction with the input signal to retrieve the filter function.
'''
from brian import *
from brian.hears import *
from numpy.random import randn
from matplotlib import pyplot

dt = 0.1*ms
defaultclock.dt = dt
stimulus_duration = 15000*ms
stimulus = randn(int(stimulus_duration/     dt))

#filter
n=200
filt = exp(-((linspace(0.5,n,n))-(n+5)/2)**2/(n/3));
filt = filt/norm(filt)*1000;
filtered_stimulus = convolve(stimulus,filt)


neuron = PoissonGroup(1,lambda t:filtered_stimulus[int(t/dt)])

spikes = SpikeMonitor(neuron)
run(stimulus_duration,report='text')
spikes = spikes[0] #resulting spikes

max_interval = 20*ms #window duration of the spike triggered average
onset = 10*ms
sta,time_axis = spike_triggered_average(spikes,stimulus,max_interval,dt,onset=onset,display=True)


figure()
plot(time_axis,filt/max(filt))
plot(time_axis,sta/max(sta))
xlabel('time axis')
ylabel('sta')
legend(('real filter','estimated filter'))

show()



from brian import *

dt = 0.1*ms
N = 500

G = NeuronGroup(1, 'V:1')

G.V = TimedArray(linspace(0, 1, N), dt=dt)
M = StateMonitor(G, 'V', record=True)

net = MagicNetwork()
net.prepare()
try:
    print net.clocks
except:
    print net.clock, net.clock.dt

net.run(N*dt)

V = M[0]

subplot(211)
plot(V)
subplot(212)
plot(diff(V))
show()



'''
The Bufferable class serves as a base for all the other Brian.hears classes
'''
from numpy import zeros, empty, hstack, vstack, arange, diff

class Bufferable(object):
    '''
    Base class for Brian.hears classes
    
    Defines a buffering interface of two methods:
    
    ``buffer_init()``
        Initialise the buffer, should set the time pointer to zero and do
        any other initialisation that the object needs.
        
    ``buffer_fetch(start, end)``
        Fetch the next samples ``start:end`` from the buffer. Value returned
        should be an array of shape ``(end-start, nchannels)``. Can throw an
        ``IndexError`` exception if it is outside the possible range.
        
    In addition, bufferable objects should define attributes:
    
    ``nchannels``
        The number of channels in the buffer.
        
    ``samplerate``
        The sample rate in Hz.
        
    By default, the class will define a default buffering mechanism which can
    easily be extended. To extend the default buffering mechanism, simply
    implement the method:
    
    ``buffer_fetch_next(samples)``
        Returns the next ``samples`` from the buffer.
    
    The default methods for ``buffer_init()`` and ``buffer_fetch()`` will
    define a buffer cache which will get larger if it needs to to accommodate
    a ``buffer_fetch(start, end)`` where ``end-start`` is larger than the
    current cache. If the filterbank has a ``minimum_buffer_size`` attribute,
    the internal cache will always have at least this size, and the
    ``buffer_fetch_next(samples)`` method will always get called with
    ``samples>=minimum_buffer_size``. This can be useful to ensure that the
    buffering is done efficiently internally, even if the user request
    buffered chunks that are too small. If the filterbank has a
    ``maximum_buffer_size`` attribute then ``buffer_fetch_next(samples)`` will
    always be called with ``samples<=maximum_buffer_size`` - this can be useful
    for either memory consumption reasons or for implementing time varying
    filters that need to update on a shorter time window than the overall
    buffer size.
    
    The following attributes will automatically be maintained:
    
    ``self.cached_buffer_start``, ``self.cached_buffer_end``
        The start and end of the cached segment of the buffer
    
    ``self.cached_buffer_output``
        An array of shape ``((cached_buffer_end-cached_buffer_start, nchannels)``
        with the current cached segment of the buffer. Note that this array can
        change size.
    '''
    def buffer_fetch(self, start, end):
        if not hasattr(self, 'cached_buffer_start'):
            self.buffer_init()
        # optimisations for the most typical cases, which are when start:end is
        # the current cached segment, or when start:end is the next cached
        # segment of the same size as the current one
        if start==self.cached_buffer_start and end==self.cached_buffer_end:
            return self.cached_buffer_output
        if start==self.cached_buffer_end and end-start==self.cached_buffer_output.shape[0]:
            self.cached_buffer_output = self._buffer_fetch_next(end-start)
            self.cached_buffer_start = start
            self.cached_buffer_end = end
            return self.cached_buffer_output
        # handle bad inputs
        if end<start:
            raise IndexError('Buffer end should be larger than start.')
        if start<self.cached_buffer_start:
            raise IndexError('Attempted to fetch output that has disappeared from the buffer.')
        # If the requested segment of the buffer is entirely within the cache,
        # just return it.
        if end<=self.cached_buffer_end:
            bstart = start-self.cached_buffer_start
            bend = end-self.cached_buffer_start
            return self.cached_buffer_output[bstart:bend, :]
        # Otherwise we need to fetch some new samples.
        # in case of minimum_buffer_size, we need to remember what the
        # requested end point was, because with a minimum buffer size we need
        # to extend the end point
        req_end = end 
        samples = end-self.cached_buffer_end
        if hasattr(self, 'minimum_buffer_size'):
            samples = max(samples, self.minimum_buffer_size)
            end = self.cached_buffer_end+samples
        newsegment = self._buffer_fetch_next(samples)
        # otherwise we have an overlap situation - I guess this won't really
        # happen very often but it has to be handled correctly in case.
        new_end = end
        # if end-start is longer than the size of the current cached buffer, we
        # will have to increase the size of the cache
        new_start = min(new_end-self.cached_buffer_output.shape[0], start)
        new_size = new_end-new_start
        new_output = empty((new_size, self.nchannels))
        if samples!=new_size:
            new_output[:new_size-samples, :] = self.cached_buffer_output[samples-new_size:, :]
        new_output[new_size-samples:, :] = newsegment
        self.cached_buffer_output = new_output
        self.cached_buffer_start = new_start
        self.cached_buffer_end = new_end
        # return only those values up to the requested end point
        return new_output[start-self.cached_buffer_start:req_end-self.cached_buffer_start, :]
    
    def _buffer_fetch_next(self, samples):
        # This method checks if there is a maximum buffer size, and if so
        # splits the buffer_fetch_next into multiple pieces of at most this size
        if not hasattr(self, 'maximum_buffer_size'):
            return self.buffer_fetch_next(samples)
        bufsize = self.maximum_buffer_size
        endpoints = hstack((arange(0, samples, bufsize), samples))
        sizes = diff(endpoints)
        return vstack(tuple(self.buffer_fetch_next(size) for size in sizes))
    
    def buffer_init(self):
        self.cached_buffer_output = zeros((0, self.nchannels))
        self.cached_buffer_start = 0
        self.cached_buffer_end = 0
    
    def buffer_fetch_next(self, samples):
        raise NotImplementedError
    
    nchannels = NotImplemented
    samplerate = NotImplemented



'''
Tutorial 1b: Counting spikes
****************************

In the previous part of the tutorial we looked at the following:

* Importing the Brian module into Python
* Using quantities with units
* Defining a neuron model by its differential equation
* Creating a group of neurons
* Running a network

In this part, we move on to looking at the output of the network.
    
The first part of the code is the same.
'''
from brian import *

tau = 20 * msecond        # membrane time constant
Vt = -50 * mvolt          # spike threshold
Vr = -60 * mvolt          # reset value
El = -60 * mvolt          # resting potential (same as the reset)

G = NeuronGroup(N=40, model='dV/dt = -(V-El)/tau : volt',
              threshold=Vt, reset=Vr)
'''
Counting spikes
~~~~~~~~~~~~~~~

Now we would like to have some idea of what this network is
doing. In Brian, we use monitors to keep track of the behaviour
of the network during the simulation. The simplest monitor of
all is the :class:`SpikeMonitor`, which just records the spikes from a
given :class:`NeuronGroup`.
'''
M = SpikeMonitor(G)
'''
Results
~~~~~~~

Now we run the simulation as before:
'''
run(1 * second)
'''
And finally, we print out how many spikes there were:
'''
print M.nspikes
'''
So what's going on? Why are there 40 spikes? Well, the answer is
that the initial value of the membrane potential for every neuron
is 0 mV, which is above the threshold potential of -50 mV and so there
is an initial spike at t=0 and then it resets to -60 mV and stays there,
below the threshold potential. In the next part of this tutorial, we'll
make sure there are some more spikes to see.
'''



from sparsematrix import *
from connectionvector import *
from constructionmatrix import *
from connectionmatrix import *
from connection import *
from delayconnection import *
from otherconnections import *



# -*- coding:utf-8 -*-
from brian import *
from brian.library.IF import *
import numpy

# Parameters
area = 20000 * umetre ** 2
Cm = (20 * ufarad * cm ** -2) * area
# Time constants
taue = 5 * ms
taui = 10 * ms
#Maximal conductances
gK = (8.0 * usiemens * cm ** -2) * area
gCa = (4.4 * usiemens * cm ** -2) * area #type II
gL = (2.0 * usiemens * cm ** -2) * area
# Reversal potentials
EK = -80 * mV
ECa = -60 * mV
EL = 120 * mV
Ee = 1 * mV#0*mV
Ei = -80 * mV
v1 = -1.2 * mV
v2 = 18 * mV
v3 = 2 * mV
v4 = 30 * mV

we = 30 * nS ##6# excitatory synaptic weight (voltage)
wi = 67 * nS # inhibitory synaptic weight

phi = 0, 04 * 1 # that's the typo: should be 0.04*1
Vr = -70 * mV
threshold = 48 * mV#-30

# Model
##Morris-Lecar##
eqsRS = Equations('''
dvm/dt=(gL*(EL-vm)+ gK*w/mV*(EK-vm)+ gCa*minf*(ECa-vm) +(ge*(Ee-vm)+gi*(Ei-vm)))/Cm + I:volt
dw/dt= phi*(winf*mV-w)/tauw: volt
minf = 0.5*(1+exp(2*((vm-v1)/v2))-1)/(exp(2*((vm-v1)/v2))+1):1
winf = 0.5*(1+exp(2*((vm-v3)/v4))-1)/(exp(2*((vm-v3)/v4))+1):1
tauw = 1*ms/((exp((vm-v3)/v4)+exp(-(vm-v3)/v4))/2):second
dge/dt = -ge*(1./taue): siemens
dgi/dt = -gi*(1./taui): siemens
I: volt/second
''')
group = NeuronGroup(2, model=eqsRS, method='Euler')



from brian import *


class RandomEventClock(EventClock):
    def __init__(self, t=0 * ms, dt=1 * second, randomfunc=None):
        EventClock.__init__(self, t=t, dt=dt)
        if randomfunc is None:
            randomfunc = rand
        self.randomfunc = randomfunc

    def tick(self):
        self._t += self.randomfunc()*self._dt

G = NeuronGroup(1, 'V:1')
M = StateMonitor(G, 'V', record=True)

@network_operation(clock=RandomEventClock())
def f():
    G.V += 1

run(5 * second)

M.plot()
show()



from brian.hears import *
import brian.hears.filtering as filtering
from numpy import ones, int32
import multiprocessing

import pycuda
#import pycuda.autoinit as autoinit
import pycuda.driver as drv
import pycuda.gpuarray as gpuarray
from pycuda.compiler import SourceModule



import os, sys
import atexit

#def set_gpu_device(n):
#    """
#    This function makes pycuda use GPU number n in the system.
#    """
#    try:
#        pycuda.context.detach()
#    except:
#        pass
#    pycuda.context = drv.Device(n).make_context()
#
#def close_cuda():
#    """
#    Closes the current context. MUST be called at the end of the script.
#    """
#    if pycuda.context is not None:
#        try:
#            pycuda.context.pop()
#            pycuda.context = None
#        except:
#            pass
#
#atexit.register(close_cuda)

def doit(x):

   # drv.init()
#    print id(pycuda.context)

    filtering.set_gpu_device(x)
#    print 'a'
#    pycuda.context = drv.Device(0).make_context()
#    print 'b'
    sys.stdin = file(os.devnull)
    sys.stdout = file(os.devnull)

    code = '''
    __global__ void test(double *x, int n)
    {
     int i = blockIdx.x * blockDim.x + threadIdx.x;
     if(i>=n) return;
     x[i] *= 2.0;
    }
    '''

    mod = SourceModule(code)
    f = mod.get_function('test')
    x = gpuarray.to_gpu(ones(100))
    f(x, int32(100), block=(100, 1, 1))
    y = x.get()
    
    filtering.close_cuda()
    
    return y

if __name__ == '__main__':
#    pycuda.context=None
    #import sys
#    print 'A'
#    set_gpu_device(0)
#    print 'B'
#    filtering.set_gpu_device(0)
#    print 'C'
#    filtering.set_gpu_device(0)
#    print 'D'

    #drv.init()
    #drv.init()

    #doit(0)
    #print id(pycuda.context)
    #close_cuda()

    pool = multiprocessing.Pool(2)
    result = pool.map(doit, [0,0])
    print result



from brian import *

def ircam_plot_count(hrtfset, count, index=None, showbest=True, absolute=False,
                     vmin=None, vmax=None, I=None, ms=20, mew=2, indexcol='k', bestcol='w'):
    if I is None: I = arange(len(count))
    count = array(count, dtype=float)
    num_indices = hrtfset.num_indices
    count.shape = (count.size/num_indices, num_indices)
    count = sum(count, axis=0)
    img = zeros((10, 24))
    for i, c in enumerate(count):
        if i in I:
            elev = hrtfset.coordinates['elev'][i]
            azim = hrtfset.coordinates['azim'][i]
            if elev<60:
                w = 1
            elif elev==60:
                w = 2
            elif elev==75:
                w = 4
            elif elev==90:
                w = 24
                azim = -180
            if azim>=180: azim -= 360
            x = int((azim+180)/15)
            y = int((elev+45)/15)
            img[y, x:x+w] = c
    if absolute:
        imshow(img, origin='lower left', interpolation='nearest', extent=(-180-7.5, 180-7.5, -45-7.5, 90+7.5),
               vmin=vmin, vmax=vmax)
        axis('tight')
    else:
        imshow(img, origin='lower left', interpolation='nearest', extent=(-180-7.5, 180-7.5, -45-7.5, 90+7.5))
        axis('tight')
    if index is not None:
        azim = hrtfset.coordinates['azim'][index]
        elev = hrtfset.coordinates['elev'][index]
        if azim>=180: azim -= 360
        plot([azim], [elev], '+', ms=ms, mew=mew, color=indexcol)
    if showbest:
        i = argmax(count)
        azim = hrtfset.coordinates['azim'][i]
        elev = hrtfset.coordinates['elev'][i]
        if azim>=180: azim -= 360
        plot([azim], [elev], 'x', ms=ms, mew=mew, color=bestcol)
    return img      


from brian import *
set_global_preferences(usenewbrianhears=True)
from brian.hears import *

#x = Sound((whitenoise(100*ms), whitenoise(100*ms)))

#x[:50*ms] = 0.1*Sound((whitenoise(50*ms), whitenoise(50*ms)))
#x[:50*ms, 0] = 0.1*whitenoise(50*ms)

x = whitenoise(100*ms)*Sound(linspace(0, 1, int(100*ms*(44.1*kHz))))#[:100*ms]
x = x[::-1]
#x[:50*ms] = 0.1*whitenoise(50*ms)

plot(x[:,0])
show()



"""This module is used by the package Brian for turning units on and off.

For the user, you can ignore this unless you want to turn units off,
in which case you simply put at the top of your program (before
importing anything from brian):

from brian_unit_prefs import turn_off_units
turn_off_units()

Or to turn units off and not have a warning printed:

from brian_unit_prefs import turn_off_units
turn_off_units(warn=False)
"""


class _unit_prefs():
    pass

bup = _unit_prefs()

bup.use_units = True
bup.warn_about_units = True

def turn_off_units(warn=True):
    bup.use_units = False
    bup.warn_about_units = warn



'''
Uses the :func:`~brian.tools.taskfarm.run_tasks` function to run a task on
multiple CPUs and save the results to a
:class:`~brian.tools.datamanager.DataManager` object.
'''
from brian import *
from brian.tools.datamanager import *
from brian.tools.taskfarm import *

def find_rate(k, report):
    eqs = '''
    dV/dt = (k-V)/(10*ms) : 1
    '''
    G = NeuronGroup(1000, eqs, reset=0, threshold=1)
    M = SpikeCounter(G)
    run(30*second, report=report)
    return (k, mean(M.count)/30)

if __name__=='__main__':
    N = 20
    dataman = DataManager('taskfarmexample')
    if dataman.itemcount()<N:
        M = N-dataman.itemcount()
        run_tasks(dataman, find_rate, rand(M)*19+1)
    X, Y = zip(*dataman.values())
    plot(X, Y, '.')
    xlabel('k')
    ylabel('Firing rate (Hz)')
    show()



'''
Hansel et al (1998)
On Numerical Simulations of Integrate-and-Fire Neural Networks.
Neural Computation

The authors show that a very small time step is necessary to
obtain some synchronization results.
The model is a network of coupled oscillators (all-to-all).
'''
from brian import *

#defaultclock.dt=.01*ms
N = 128
duration1 = 2 * second
duration2 = 2 * second
Vl = -60 * mV
theta = -40 * mV
tau = 10 * ms
tau1 = 3 * ms
tau2 = 1 * ms
I0 = 21 * mV # I couldn't find the value in the text!
# there is also an error in the text: replace tau*I0 by I0
w = .7 * mV # slightly below critical value
eqs = '''
dV/dt=(Vl-V+I+I0)/tau : volt
dI/dt=(x-I)/tau1 : volt
dx/dt=-x/tau2 : volt
'''

T = -tau * log((I0 - theta) / (I0 - Vl))

P = NeuronGroup(N, model=eqs, threshold=theta, reset=Vl)
C = Connection(P, P, 'x', weight=w)
c = 1. # in 0..1
P.V = I0 + (Vl - I0) * exp(-c * arange(N) / N * T / tau)

run(duration1)

M = PopulationRateMonitor(P) # average spiking activity
S = StateMonitor(P, 'V') # to measure the individual variances of V

mv = []
@network_operation
def monitor_averageV():
    mv.append(mean(P.V))

run(duration2)
deltaN = var(mv)
delta = mean(S.var)
sigma = deltaN / delta # Measure of synchrony
print sigma

# Visual check
plot(M.times / ms, M.smooth_rate(1 * ms))
show()



from brian import *
from numpy import *

__all__ = ['CoincidenceDetectorGroup']


class CoincidenceDetectorConnection(Connection):
    def propagate(self, spikes):
        self.target.LSST[spikes] = self.source.clock._t


class CoincidenceDetectorStateUpdater(LazyStateUpdater):
    def __init__(self, allow_multiple, ordered):
        self.allow_multiple = allow_multiple
        self.ordered = ordered
        LazyStateUpdater.__init__(self)

    def __call__(self, P):
        lsst = P.LSST
        i0 = P.indices[0]
        i1 = P.indices[1]
        lssti0 = lsst[i0]
        lssti1 = lsst[i1]
        firing = abs(lssti0 - lssti1) < P.delta
        if self.ordered:
            firing = logical_and(firing, lssti0 > lssti1)
        firing = logical_and(firing, lssti0 > P.lastfiring)
        firing = logical_and(firing, lssti1 > P.lastfiring)
        P._S[0][firing] = 1
        if self.allow_multiple:
            # this allows each spike to be used for multiple coincidences
            P.lastfiring[firing] = where(lssti0 > lssti1, lssti0, lssti1) - P.clock._dt * 0.5
        else:
            # this allows each spike to be used for multiple coincidences
            P.lastfiring[firing] = P.clock._t - P.clock._dt * 0.5


class CoincidenceDetectorGroup(NeuronGroup):
    '''
    Perfect pairwise coincidence detector group
    
    Initialisation arguments:
    
    ``source``
        The source group
    ``N``
        The number of coincidence detector neurons
    ``delta``
        The maximum time interval to count as a coincidence
    ``allow_multiple``
        Whether or not a spike can be used for multiple coincidences
        for a given coincidence detector. Suppose neuron 0 fires at
        time 1ms and neuron 1 fires at times 2ms and 3ms with window
        5ms say. If ``allow_multiple=True`` then the coincidence
        detector will fire at times 2ms and 3ms, if ``allow_multiple=False``
        then it will only fire at time 2ms.
    ``ordered``
        If ``False`` and s, t are the spike times then ``abs(s-t)<delta``
        is the coincidence condition, if ``True`` then ``s>t`` must also
        hold.
    
    .. method: set_indices
    
        Takes two arguments, ``i0`` and ``i1`` which should be lists or
        arrays of length the number of neurons in the group. Coincidence
        detector neuron ``k`` receives inputs from ``i0[k]`` and ``i1[k]``
        in the source group.
    '''
    def __init__(self, source, N, delta=1 * ms, allow_multiple=False, ordered=False, clock=None, **kwds):
        if clock is None:
            clock = source.clock
        NeuronGroup.__init__(self, N, model=CoincidenceDetectorStateUpdater(allow_multiple, ordered), reset=0, threshold=0.5, clock=clock, **kwds)
        self._S[:] = 0
        self.source = source
        self.conn = CoincidenceDetectorConnection(source, self)
        self.contained_objects = [self.conn]
        self.LSST = zeros(len(source))
        self.lastfiring = ones(N) * clock._dt * 0.5
        if isinstance(delta, Quantity):
            delta = float(delta)
        self.delta = delta
        self.indices = zeros((2, N), dtype=int)

    def set_indices(self, i0, i1):
        self.indices[0] = i0
        self.indices[1] = i1

    def reinit(self):
        self.LSST[:] = 0
        self.lastfiring[:] = self.clock._dt * 0.5

if __name__ == '__main__':
    if 1:
        G = MultipleSpikeGeneratorGroup(
                [
                 [1 * ms, 2 * ms, 2.1 * ms],
                 [1.5 * ms]
                ])
        H = CoincidenceDetectorGroup(G, 2, 1 * ms, ordered=True, allow_multiple=True)
        H.indices[0] = [0, 1]
        H.indices[1] = [1, 0]
        MG = SpikeMonitor(G)
        MH = SpikeMonitor(H)
        run(10 * ms)
        print MG.spikes
        print MH.spikes
    if 0:
        G = MultipleSpikeGeneratorGroup(
                [
                 [1 * ms, 2.5 * ms],
                 [1.5 * ms]
                ])
        H = CoincidenceDetectorGroup(G, 2)
        H.indices[0] = [0, 1]
        H.indices[1] = [1, 0]
        MG = SpikeMonitor(G)
        MH = SpikeMonitor(H)
        run(10 * ms)
        print MG.spikes
        print MH.spikes
    if 0:
        G = PoissonGroup(2, 250 * Hz)
        H = CoincidenceDetectorGroup(G, 1)
        H.set_indices([0], [1])
        MG = SpikeMonitor(G)
        MH = SpikeMonitor(H)
        run(1 * second)
        raster_plot(MG, MH)
        show()



import sys
import time
import threading
from itertools import izip, count

__all__ = ['foreach', 'parallel_map']

def foreach(f, l, threads=3, return_=False):
    """
    Apply f to each element of l, in parallel
    """

    if threads > 1:
        iteratorlock = threading.Lock()
        exceptions = []
        if return_:
            n = 0
            d = {}
            i = izip(count(), l.__iter__())
        else:
            i = l.__iter__()


        def runall():
            while True:
                iteratorlock.acquire()
                try:
                    try:
                        if exceptions:
                            return
                        v = i.next()
                    finally:
                        iteratorlock.release()
                except StopIteration:
                    return
                try:
                    if return_:
                        n, x = v
                        d[n] = f(x)
                    else:
                        f(v)
                except:
                    e = sys.exc_info()
                    iteratorlock.acquire()
                    try:
                        exceptions.append(e)
                    finally:
                        iteratorlock.release()

        threadlist = [threading.Thread(target=runall) for j in xrange(threads)]
        for t in threadlist:
            t.start()
        for t in threadlist:
            t.join()
        if exceptions:
            a, b, c = exceptions[0]
            raise a, b, c
        if return_:
            r = d.items()
            r.sort()
            return [v for (n, v) in r]
    else:
        if return_:
            return [f(v) for v in l]
        else:
            for v in l:
                f(v)
            return

def parallel_map(f, l, threads=3):
    return foreach(f, l, threads=threads, return_=True)

if __name__ == '__main__':
    def f(x):
        print x
        time.sleep(0.5)
    foreach(f, range(10))
    def g(x):
        time.sleep(0.5)
        print x
        raise ValueError, x
        time.sleep(0.5)
    foreach(g, range(10))



import pycuda.autoinit as autoinit
import pycuda.driver as drv
from pycuda.gpuarray import GPUArray
from pycuda import gpuarray
import bisect
import numpy, pylab, time
from itertools import repeat
import cProfile as profile
import pstats

N = 512 * 3200
blocksize = 512
duration = 10000
record = False
doprofile = False
if drv.get_version() == (2, 0, 0): # cuda version
    precision = 'float'
elif drv.get_version() > (2, 0, 0):
    precision = 'double'
else:
    raise Exception, "CUDA 2.0 required"

if precision == 'double':
    mydtype = numpy.float64
else:
    mydtype = numpy.float32

block = (blocksize, 1, 1)
grid = (N / blocksize, 1)
Ne = int(N * 0.8)
Ni = N - Ne

mod = drv.SourceModule("""
__global__ void stateupdate(SCALAR *V_arr, SCALAR *ge_arr, SCALAR *gi_arr)
{
    int i = blockIdx.x * blockDim.x + threadIdx.x;
    SCALAR V = V_arr[i];
    SCALAR ge = ge_arr[i];
    SCALAR gi = gi_arr[i]; 
    SCALAR V__tmp = (ge+gi-(V+0.049))/0.02;
    SCALAR ge__tmp = -ge/0.005;
    SCALAR gi__tmp = -gi/0.01;
    V_arr[i] = V+0.0001*V__tmp;
    ge_arr[i] = ge+0.0001*ge__tmp;
    gi_arr[i] = gi+0.0001*gi__tmp;
}

__global__ void threshold(SCALAR *V, int *spikes, bool *spiked, unsigned int *global_j, int N)
{
    int i = blockIdx.x * blockDim.x + threadIdx.x;
    bool this_spiked = V[i]>-0.05; 
    spiked[i] = this_spiked;
    if(this_spiked) // && i<N) // can leave out i<N check if N%blocksize=0
    {
        unsigned int j = atomicInc(global_j, N);
        spikes[j] = i;
    }
}

__global__ void propagate(int *spikes, int numspikes, SCALAR *v, SCALAR *W, int N)
{
    int i = blockIdx.x * blockDim.x + threadIdx.x;
    SCALAR v_tmp = v[i];
    for(int j=0; j<numspikes; j++)
        v_tmp += W[i+N*spikes[j]];
    v[i] = v_tmp;
}

__global__ void reset(SCALAR *V, bool *spiked)
{
    int i = blockIdx.x * blockDim.x + threadIdx.x;
    bool has_spiked = spiked[i];
    V[i] = (V[i]*!has_spiked)+(-0.06)*has_spiked; // i.e. V[i]=-0.06 if spiked[i]
}
""".replace('SCALAR', precision))
stateupdate = mod.get_function("stateupdate")
threshold = mod.get_function("threshold")
propagate = mod.get_function("propagate")
reset = mod.get_function("reset")

V = gpuarray.to_gpu(numpy.array(numpy.random.rand(N) * 0.01 - 0.06, dtype=mydtype))
ge = gpuarray.to_gpu(numpy.zeros(N, dtype=mydtype))
gi = gpuarray.to_gpu(numpy.zeros(N, dtype=mydtype))

#we = 0.00162
#wi = -0.009
#pconn = 80./N
#CeW = numpy.array((numpy.random.rand(Ne, N)<pconn)*we,dtype=mydtype)
#CiW = numpy.array((numpy.random.rand(Ni, N)<pconn)*wi,dtype=mydtype)
#CeW = gpuarray.to_gpu(CeW)
#CiW = gpuarray.to_gpu(CiW)
#
#gpu_spikes = drv.mem_alloc(4*N)
#gpu_spikes_exc = drv.mem_alloc(4*N)
#gpu_spikes_inh = drv.mem_alloc(4*N)
#gpu_spiked = gpuarray.to_gpu(numpy.zeros(N, dtype=bool))
#gpu_spike_index = drv.mem_alloc(4)
#master_spikes = numpy.zeros(N, dtype=int)
#spike_index = numpy.zeros(1, dtype=numpy.uint32)
#drv.memcpy_htod(gpu_spikes, master_spikes)
#drv.memcpy_htod(gpu_spike_index, spike_index)
#
#if record:
#    recspikes = []
#    recv0 = numpy.zeros(duration,dtype=mydtype)
#totalnspikes = 0

stateupdate.prepare(('i', 'i', 'i'), block)
#threshold.prepare(('i', 'i', 'i', 'i', numpy.int32), block)
#propagate.prepare(('i', numpy.int32, 'i', 'i', numpy.int32), block)
#reset.prepare(('i', 'i'), block)

stateupdate_args = (grid, int(V.gpudata), int(ge.gpudata), int(gi.gpudata))
#threshold_args = (grid, int(V.gpudata), int(gpu_spikes), int(gpu_spiked.gpudata), int(gpu_spike_index), numpy.int32(N))
#reset_args = (grid, int(V.gpudata), int(gpu_spiked.gpudata))

def run_sim():
    global totalnspikes
    for t in xrange(duration):
        #stateupdate(V, ge, gi, block=block, grid=grid)
        if t == 0:
            stateupdate.prepared_call(*stateupdate_args)
        else:
            stateupdate.launch_grid(*grid)
#        spike_index[0] = 0
#        drv.memcpy_htod(gpu_spike_index, spike_index)
#        #threshold(V, gpu_spikes, gpu_spiked, gpu_spike_index, numpy.int32(N), block=block, grid=grid)
#        if t==0:
#            threshold.prepared_call(*threshold_args)
#        else:
#            threshold.launch_grid(*grid)
#        drv.memcpy_dtoh(spike_index, gpu_spike_index)
#        numspikes = spike_index[0]
#        spikes = master_spikes[:numspikes]
#        drv.memcpy_dtoh(spikes, gpu_spikes)
#        spikes.sort()
#        numspikes_exc = bisect.bisect_left(spikes, Ne)
#        numspikes_inh = numspikes - numspikes_exc
#        drv.memcpy_htod(gpu_spikes_exc, spikes[:numspikes_exc])
#        drv.memcpy_htod(gpu_spikes_inh, spikes[numspikes_exc:]-Ne)
#        ###propagate(gpu_spikes_exc, numpy.int32(numspikes_exc), ge, CeW, numpy.int32(N), block=block, grid=grid)
#        propagate.prepared_call(grid, int(gpu_spikes_exc), numpy.int32(numspikes_exc), int(ge.gpudata), int(CeW.gpudata), numpy.int32(N))
#        ###propagate(gpu_spikes_inh, numpy.int32(numspikes_inh), gi, CiW, numpy.int32(N), block=block, grid=grid)
#        propagate.prepared_call(grid, int(gpu_spikes_inh), numpy.int32(numspikes_inh), int(gi.gpudata), int(CiW.gpudata), numpy.int32(N))
#        ###reset(V, gpu_spiked, block=block, grid=grid)
#        if t==0:
#            reset.prepared_call(*reset_args)
#        else:
#            reset.launch_grid(*grid)
#        if record:
#            recspikes += zip(spikes, repeat(t))
#            drv.memcpy_dtoh(recv0[t:t+1], V.gpudata)
#        totalnspikes += numspikes

if doprofile:
    start = time.time()
    profile.run('run_sim()', 'cudacuba.prof')
    end = time.time()
    stats = pstats.Stats('cudacuba.prof')
    #stats.strip_dirs()
    stats.sort_stats('cumulative', 'calls')
    stats.print_stats(50)
else:
    start = time.time()
    run_sim()
    end = time.time()
print 'N:', N
print 'GPU time:', end - start
print 'Number of spikes:', totalnspikes

if record:
    i, t = zip(*recspikes)
    pylab.subplot(121)
    pylab.plot(t, i, '.')
    pylab.subplot(122)
    pylab.plot(recv0)
    pylab.show()



'''
An example with correlated spike trains
From: Brette, R. (2007). Generation of correlated spike trains.
'''
from brian import *

N = 100
c = .2
nu = linspace(1*Hz, 10*Hz, N)
P = c*dot(nu.reshape((N,1)), nu.reshape((1,N)))/mean(nu**2)
tauc = 5*ms

spikes = mixture_process(nu, P, tauc, 1*second)
input = SpikeGeneratorGroup(N, spikes)

S = SpikeMonitor(input)
run(1000 * ms)

raster_plot(S)
show()



'''
Ionic currents for Brian
'''
from brian.units import check_units, siemens, volt
from brian.membrane_equations import Current
from brian.equations import unique_id

@check_units(El=volt)
def leak_current(gl, El, current_name=None):
    '''
    Leak current: gl*(El-vm)
    '''
    current_name = current_name or unique_id()
    return Current('I=gl*(El-vm) : amp', gl=gl, El=El, I=current_name, current_name=current_name)

#check_units(EK=volt)
def K_current_HH(gmax, EK, current_name=None):
    '''
    Hodkin-Huxley K+ current.
    Resting potential is 0 mV.
    '''
    current_name = current_name or unique_id()
    return Current('''
    I=gmax*n**4*(EK-vm) : amp
    dn/dt=alphan*(1-n)-betan*n : 1
    alphan=.01*(10*mV-vm)/(exp(1-.1*vm/mV)-1)/mV/ms : Hz
    betan=.125*exp(-.0125*vm/mV)/ms : Hz
    ''', gmax=gmax, EK=EK, I=current_name, current_name=current_name)
    # 2 problems here:
    # * The current variable is determined thanks to the units,
    #   so it doesn't work if units are off.
    # * Variables n, alphan and betan may conflict with other variables
    #   (e.g. in other currents).
    #
    # Solutions:
    # * Set current_name explicitly. If None, generate an id or use I_K.
    # * Anonymise names with unique id, on option. This could be an option in
    #   Current().

#check_units(ENa=volt)
def Na_current_HH(gmax, ENa, current_name=None):
    '''
    Hodkin-Huxley Na+ current.
    '''
    current_name = current_name or unique_id()
    return Current('''
    I=gmax*m**3*h*(ENa-vm) : amp
    dm/dt=alpham*(1-m)-betam*m : 1
    dh/dt=alphah*(1-h)-betah*h : 1
    alpham=.1*(25*mV-vm)/(exp(2.5-.1*vm/mV)-1)/mV/ms : Hz
    betam=4*exp(-.0556*vm/mV)/ms : Hz
    alphah=.07*exp(-.05*vm/mV)/ms : Hz
    betah=1./(1+exp(3.-.1*vm/mV))/ms : Hz
    ''', gmax=gmax, ENa=ENa, I=current_name, current_name=current_name)



'''
A different way of plotting raster plots
'''
from brian import *

tau = 5 * ms
N = 10
f = 50 * Hz
a = .2

eqs = '''
dv/dt=(v0-v+a*(sin(2*pi*f*t)+sin(4*pi*f*t)))/tau : 1
v0 : 1
'''

neurons = NeuronGroup(N, model=eqs, threshold=1, reset=0)
neurons.v0 = 1.37
neurons.v = rand(N)

run(1 * second)
S = SpikeMonitor(neurons)
run(100 * ms)
#raster_plot(S)
for i, t in S.spikes:
    plot([t, t], [i, i + .9], 'b', linewidth=2)
show()



"""
Listens to AER packets through UDP.

Format:
int32 sequenceNumber
int32 address0
int32 timestamp0
int32 address1
int32 timestamp2
etc
"""
from scipy import *
import socket
from struct import *
from time import time
from brian.experimental.neuromorphic import *

UDP_IP="localhost"
UDP_PORT=8991

sock = socket.socket( socket.AF_INET, # Internet
                      socket.SOCK_DGRAM ) # UDP
sock.bind( (UDP_IP,UDP_PORT) )

first_time=True
t0=1e12
while time()-t0<5.: # run for 5 s
    data, _ = sock.recvfrom( 63000 ) # buffer size
    packet_number=unpack('>I',data[:4])
    # can we vectorize?
    # events are (address, timestamp)
    #events=[unpack('>II',data[(8*n+4):(8*n+12)]) for n in range(len(data)/8-1)]
    events=fromstring(data[4:],int32).newbyteorder('>')
    # something's wrong with the times
    
    if (len(events)>0):
        addr=events[::2]
        timestamp=events[1::2]
        if first_time: # Synchronize event timestamps and clock
            first_time=False
            t0=time()
            #ts0=events[0][1]
            ts0=timestamp[0]
        #ts=events[0][1]-ts0
        ts=timestamp[0]-ts0
        # Show first timestamp, clock time, first event
        print ts*1e-6,time()-t0,extract_DVS_event(addr[0])#,timestamp[0] #events[0][0])
    
sock.close()



'''
A very simple example Brian script to show how to implement
an integrate and fire model. In this example, we also
drive the single integrate and fire neuron with
regularly spaced spikes from the :class:`SpikeGeneratorGroup`.
'''
from brian import *

tau = 10 * ms
Vr = -70 * mV
Vt = -55 * mV

G = NeuronGroup(1, model='V:volt', threshold=Vt, reset=Vr)

input = SpikeGeneratorGroup(1, [(0, t * ms) for t in linspace(10, 100, 25)])

C = Connection(input, G)
C[0, 0] = 2 * mV

M = StateMonitor(G, 'V', record=True)

G.V = Vr
run(100 * ms)
plot(M.times / ms, M[0] / mV)
show()



"""
A model with depolarizing after-potential.
"""
from brian import *

v0=20.5*mV
eqs = '''
dv/dt = (v0-v)/(30*ms) : volt # the membrane equation
dAP/dt = -AP/(3*ms) : volt # the after-potential
vm = v+AP : volt # total membrane potential
'''
IF = NeuronGroup(1, model=eqs, threshold='vm>20*mV',
                 reset='v=0*mV; AP=10*mV')
Mv = StateMonitor(IF, 'vm', record=True)

run(500 * ms)
plot(Mv.times / ms, Mv[0] / mV)
show()



'''
In this script I test the synapses initialization/construction of Synapses
'''
from brian import *
import dev.ideas.synapses.synapses as syn
reload(syn)
log_level_debug()

gin = NeuronGroup(10, model = 'dv/dt = -v/(5*ms) : 1')
gout = NeuronGroup(20, model = 'dv/dt = -v/(5*ms) : 1')

synapses = syn.Synapses(gin, gout, model = '''w : 1; z : 1''', pre = 'v += w + rand()')

synapses[0,:] = 2
print synapses
synapses.w[:,:] = 2.1
synapses.z[1,:] = 4
print synapses.w
print synapses.z

synapses.z[0, :, :] = arange(len(synapses)*2, dtype = np.float32)
print synapses.z




from brian import *
set_global_preferences(useweave=False, usecodegen=False)
tau=1*ms
sigmax=1*mV
sigmay=2*mV

# These equations don't work
#eqs="""
#dx/dt=-x/tau+sigmax*u/tau**.5 : volt
#dy/dt=-y/tau+sigmay*u/tau**.5 : volt
#u=xi : second**(-.5)
#"""
# Workaround
eqs="""
v = x+sigmax*u : volt
w = y+sigmay*u : volt
dx/dt=-x/tau : volt
dy/dt=-y/tau : volt
du/dt=xi/tau**.5 : 1
"""
neuron=NeuronGroup(1,model=eqs)
#M=MultiStateMonitor(neuron, record=True)
M=MultiStateMonitor(neuron, ['v', 'w'], record=True)
run(10*ms)
M.plot()
show()



import os

os.chdir('../../../docs_sphinx')

import brian.globalprefs as gp

main_preferences_page_text = open('reference-preferences.txt', 'r').read()
main_preferences_page_text = main_preferences_page_text[:main_preferences_page_text.find('.. INSERT_GLOBAL_PREFERENCES_HERE')]
main_preferences_page_text += '.. INSERT_GLOBAL_PREFERENCES_HERE\n\n'
main_preferences_page_text += gp.__doc__

f = open('reference-preferences.txt', 'w')
f.write(main_preferences_page_text)
f.close()



'''
Tutorial 2c: The CUBA network
*****************************

In this part of the tutorial, we set up our first serious network
that actually does something. It implements the CUBA network, Benchmark 2 from:

    Simulation of networks of spiking neurons: A review of tools and strategies (2006).
    Brette, Rudolph, Carnevale, Hines, Beeman, Bower, Diesmann, Goodman, Harris, Zirpe,
    Natschlager, Pecevski, Ermentrout, Djurfeldt, Lansner, Rochel, Vibert, Alvarez, Muller,
    Davison, El Boustani and Destexhe.
    Journal of Computational Neuroscience

This is a network of 4000 neurons, of which 3200 excitatory, and 800 inhibitory, with
exponential synaptic currents. The neurons are randomly connected with probability 0.02.
'''
from brian import *

taum = 20 * ms          # membrane time constant
taue = 5 * ms          # excitatory synaptic time constant
taui = 10 * ms          # inhibitory synaptic time constant
Vt = -50 * mV          # spike threshold
Vr = -60 * mV          # reset value
El = -49 * mV          # resting potential
we = (60 * 0.27 / 10) * mV # excitatory synaptic weight
wi = (20 * 4.5 / 10) * mV # inhibitory synaptic weight

eqs = Equations('''
        dV/dt  = (ge-gi-(V-El))/taum : volt
        dge/dt = -ge/taue            : volt
        dgi/dt = -gi/taui            : volt
        ''')
'''
So far, this has been pretty similar to the previous part, the only
difference is we have a couple more parameters, and we've added a
resting potential ``El`` into the equation for ``V``.

Now we make lots of neurons:
'''
G = NeuronGroup(4000, model=eqs, threshold=Vt, reset=Vr)
'''
Next, we divide them into subgroups. The :meth:`~NeuronGroup.subgroup` method of a
:class:`NeuronGroup` returns a new :class:`NeuronGroup` that can be used in
exactly the same way as its parent group. At the moment, the
subgrouping mechanism can only be used to create contiguous
groups of neurons (so you can't have a subgroup consisting
of neurons 0-100 and also 200-300 say). We designate the
first 3200 neurons as ``Ge`` and the second 800 as ``Gi``, these
will be the excitatory and inhibitory neurons.
'''
Ge = G.subgroup(3200) # Excitatory neurons
Gi = G.subgroup(800)  # Inhibitory neurons
'''
Now we define the connections. As in the previous part of the
tutorial, ``ge`` is the excitatory current and ``gi`` is the inhibitory
one. ``Ce`` says that an excitatory neuron can synapse onto any
neuron in ``G``, be it excitatory or inhibitory. Similarly for
inhibitory neurons. We also randomly connect ``Ge`` and ``Gi`` to the whole of ``G`` with
probability 0.02 and the weights given in the list of
parameters at the top.
'''
Ce = Connection(Ge, G, 'ge', sparseness=0.02, weight=we)
Ci = Connection(Gi, G, 'gi', sparseness=0.02, weight=wi)
'''
Set up some monitors as usual. The line ``record=0`` in the :class:`StateMonitor`
declarations indicates that we only want to record the activity of
neuron 0. This saves time and memory.
'''
M = SpikeMonitor(G)
MV = StateMonitor(G, 'V', record=0)
Mge = StateMonitor(G, 'ge', record=0)
Mgi = StateMonitor(G, 'gi', record=0)
'''
And in order to start the network off in a somewhat
more realistic state, we initialise the membrane
potentials uniformly randomly between the reset and
the threshold.
'''
G.V = Vr + (Vt - Vr) * rand(len(G))
'''
Now we run.
'''
run(500 * ms)
'''
And finally we plot the results. Just for fun, we do a rather more
complicated plot than we've been doing so far, with three subplots.
The upper one is the raster plot of the whole network, and the
lower two are the values of ``V`` (on the left) and ``ge`` and ``gi`` (on the
right) for the neuron we recorded from. See the PyLab documentation
for an explanation of the plotting functions, but note that the
:func:`raster_plot` keyword ``newfigure=False`` instructs the (Brian) function
:func:`raster_plot` not to create a new figure (so that it can be placed
as a subplot of a larger figure). 
'''
subplot(211)
raster_plot(M, title='The CUBA network', newfigure=False)
subplot(223)
plot(MV.times / ms, MV[0] / mV)
xlabel('Time (ms)')
ylabel('V (mV)')
subplot(224)
plot(Mge.times / ms, Mge[0] / mV)
plot(Mgi.times / ms, Mgi[0] / mV)
xlabel('Time (ms)')
ylabel('ge and gi (mV)')
legend(('ge', 'gi'), 'upper right')
show()
'''
.. image:: images/tutorials/2c.jpg
'''
